#ifndef __CVS_H__
#define __CVS_H__

#include "common.h"

enum cvscommandkind {
    ckcheckout,
    ckcheckoutr,
    ckcommit,
    ckimport,
    ckupdate,
    ckstatus,
    ckstatusv,
    ckadd,
    ckremove,
    cktag,
    cklog,
    cknone
};

enum cvsresultkind {
    rknotlogged,
    rkok,
    rkaborted
};

enum cvsmodificationkind {
    mknone,
    mklocal,
    mkremote,
    mklocaladded,
    mklocalremoved
};

class cvsupdated { public:
    string fname;
    cvsmodificationkind kind;

    cvsupdated(const string afname = "", const cvsmodificationkind akind = mknone);
    ~cvsupdated();

    bool operator == (const cvsmodificationkind &akind) const;
    bool operator != (const cvsmodificationkind &akind) const;
};

class cvsrelease { public:
    string name;
    vector<string> files;

    cvsrelease(const string aname = "");
    ~cvsrelease();

    bool operator == (const string &aname) const;
    bool operator != (const string &aname) const;
};

class cvslog { public:
    string revision, log, date, author;
};

class CMotorCVS {
    private:
        FILE *fpipe;
	int cpipe[2];
        string cvsroot;

        vector<cvsupdated> updlist;
        vector<cvslog> loglist;
        vector<cvsrelease> rlist;

	void readpipe(cvscommandkind ck);
	void run(string indir, string cvscommand, cvscommandkind ck);

        static int linkfile(const char *fname, const struct stat *sb, int flag);
        
    public:
        bool nocvs;
	cvsresultkind result;

	CMotorCVS();
	~CMotorCVS();

	void setrepos(string cr);
	void checkout(string module, string localdir);
	void update(string dir);
	void commit(string dir, string msg);
        void status(string dir, vector<cvsupdated> &lst);
        void statusv(string dir, vector<cvsrelease> &lst);
        void add(string dir, string fname);
        void remove(string dir, string fname, bool physically);
        void tag(string dir, string relname);
        void log(string dir, string fname, vector<cvslog> &lst);
        void import(string dir, string module);
        void getrelease(string dir, string module, string relname, string fname);
};

extern CMotorCVS cvs;

#endif
