#ifndef __DEBUG_H__
#define __DEBUG_H__

#include "common.h"
#include "debugp.h"

enum commandkind {
    ckusual,
    cknext,
    ckbreakpoint,
    ckrun,
    ckcontinue,
    ckwatch,
    ckdontanalyze,
    ckcore,
    ckstack
};

enum lrkind {
    lrunknown,
    lrexit,
    lrbreakpoint,
    lrunabletostart,
    lrsignal,
    lrgdbshut,
    lrwrongcore,
    lrcoreok,
    lrok,
    lrnotty
};

class CMotorDebugger {
    protected:
	FILE *fpr, *fpw;
        string watchplace;
            // a place where to put the strdupped "print" command result

        vector<progstackitem> stack;
        
        int gdbpid, vtno, vtfd, thisvt, line;
        string fname, resultstr;

	lrkind lastresult;

	void rungdb();
	void setbreakpoints();
        void setbreakpoint(breakpoint &b);

        string read();
        void command(string c, commandkind kind = ckusual);
        void terminated();
	void setvt();
	void backvt();
        void takelocation(string p);
        void parsestack(string stackinfo);
        void parsecore(string info);

    public:
        bool running, active;

        vector<breakpoint> bpoints;
        vector<watchexpr> watches;

	CMotorDebugger();
	~CMotorDebugger();
	
	lrkind init();

        void run();
	void loadcore(string corefname);
	void proceed(bool into);
        void updatewatch();

	void readstack(vector<progstackitem> &astack);

        void addbreakpoint(const breakpoint &bp);
        void delbreakpoint(const breakpoint bp);

        void setvar(string name, string newval);
        string getvar(string varname);

        lrkind getlastresult();
        string getresultstr();
        string getfname();

        int getfline();

        void clear();
	void done();
};

extern CMotorDebugger deb;

#endif

