#ifndef __PROJECT_H__
#define __PROJECT_H__

#include "common.h"
#include "templ.h"
#include "debugp.h"

enum makenamemode {mmdist, mmlocal};
enum createmode {cmscratch, cmfiles, cmcvs};

class screateinfo { public:
    string pname, rootdir, cvsroot, cvsmodule;
    bool gnudoc, gensource;
    motortemplate *t;
    createmode mode;
    screateinfo(): gnudoc(true), gensource(true), t(0), mode(cmscratch) {};
};

class projectfile {
    protected:
	string fname;
        filefolder *folder;
        
    public:
	projectfile(filefolder *f, string name = "");
	~projectfile();

        string getfname() const;
        filefolder *getfolder() const;
};

class installrule {
    protected:
        string fname, dest;
        
    public:
        installrule(string nfname, string ndest);
        ~installrule();

        string getfname();
        string getdest();
};

class motorproject {
    protected:
        string arootdir;
	linkedlist files, ineditor, templist, insts;

        vector<debugpoint> dpoints;

        static int gatherfilesftw(const char *file, const struct stat *sb, int flag);
        static int projectfilecompare(void *key, void *current);
        static char *svmakefile(const char *value, const char *key);
        static void exec_editload(int argc, const char **argv);

    public:
        string result;
        string rootdir;
	string name;
	string outdir;
	string cvsroot;
	string cvsmodule;
	string compoptions;
	string linkoptions;
        string args;
        string version;
        string packgendir;
        string altermake;
        string aftermake;
        string packinstcmd;

	motortemplate *tmpl;
	bool pautoconf, pconnected, paddinfo, watchactive, leavemakef;

	motorproject();
	~motorproject();

	string relativefname(string fname);
        string absfname(string fname);
        string getmakename();

        void clearmakefiles();
        void genmakefile(makenamemode mm = mmlocal);
        void genconnectedmakefiles(makenamemode mm = mmlocal);

        void populateparselist(pparamslist *params);

        void save();
	void chdir();

        int getfilecount();
        int getedcount();
        int getinstcount();

        bool cvsenabled();
        
        void addfile(string fname, filefolder *fold = 0);
        void edadd(string fname);
        void addinst(string nfname, string ndest);
        
        installrule *getinst(int n);
        projectfile *getfile(int n);
        string edget(int n);
        linkedlist *getfiles(filefolder *fold);

        void removefile(string fname, folderelement elem);
        void removefile(projectfile *pf);
        void removeinst(int n);
        void edclear();

        void *foreachfile(listforeachfunc *lff, void *arg);
        void migratefiles(motortemplate *newt);
	void gatherfiles();
        void addgnudoc();
        void generatesource(linkedlist *ngen = 0);

        void loadfromfile(string fname);

        void getdebug();
        void putdebug();

        bool isconnectedpathname(string pname);
        bool contains(string fname);
        bool modified();
};

extern motorproject *proj;

#endif
