#ifndef __MOTOR_CORE__
#define __MOTOR_CORE__

#include "common.h"
#include "cvs.h"
#include "debugp.h"

typedef void (voidfunc)();

class CMotorCore {
    protected:
        textinputline *il;
        horizontalmenu hmenu;
        verticalmenu *watchmenu;
        horizontalbar *bar;

        bool editok;
	void setsignals();

        string inputskel(string initval, string text);
        bool cvsfilltext(const vector<cvsupdated> &lst, string &text, cvsmodificationkind kind);
        
        void cvscheckoutver(string rel, string fname);
        bool cvscomment(string &ret);
        void cvslogshow(const cvslog &clog);
        void showstackitem(const progstackitem &ps);
	bool nodebask(string msg);

        static int cvscommentkeys(texteditor *ed, int k);
        static int watchkeys(verticalmenu *m, int k);
        static int windowlistkeys(verticalmenu *m, int k);
        
	static void sighandler(int signum);

    public:
	CMotorCore();
	~CMotorCore();

        string curdebfname;        // Actual debugger position
        int curdebline;

        filemanager *fm;
        textwindow *wwatch;
        int watchstart;

        string inputdir(string initval, string text);
        string inputfile(string initval, string text);
        string input(string initval, string text);

        verticalmenu *getpulldown(int n);

        int askf(string options, const char *fmt, ...);
        int ask(string options, string msg);

        void restorestate();

        void init();
        void shutdown();

        void cvscheck(cvscommandkind ck);
        void cvsgetrelease();
        void cvsfilehistory();
        void cvsimport();
        int cvsrootselect(bool emptyitem);
        void cvscommand(cvscommandkind ck);

        int execbin();
        void commandline(int argc, char *argv[]);
	void exec();
        void redraw();
	void help();

	void project_read();
	void project_write();

        void status();
        void status(int x, int y, bool imode, bool bmode);

        void bottom(string s = "");
        void bottomf(const char *fmt, ...);
        
	int execmenu(texteditor *ref);
	bool build();
        void makeclean();

        void run();
        void setargs();
        void reset();
        void debugcore();
        void debugproceed(bool into);
        void debugtocursor();
        bool checkbuildneed();
        void checkdebugresult();
        void unlight(int fn, int prevline);
        void callstack();

	void setbreakpoint();
        void markbreakpoints();
        void addwatch(string wstr);
        void updatewatches();
        void evaluate(string expr);

        void openwatch(bool activate);
        void openbreakpoints();
};

extern CMotorCore core;

#endif

