/*
* llist.h - manipulate linked lists
*
* Authors:	RomaRoma (http://www.rr.org.ua/)
*
* Created:	10.12.1999	romaroma	Initial revision.
* Changed:	06.03.2000	romaroma	new function:
*						llist_get_index();
*
*/

#ifndef _LLIST_H
#define _LLIST_H

#ifdef __cplusplus
extern "C" {
#endif 

/* Item structure */
typedef struct {
    void *prev;
    void *item;
    void *next;
} llist_item;

/* Add a new item */
llist_item *llist_add(llist_item *list, void *item);

/* Delete item specified by index */
llist_item *llist_delete(llist_item *list, int index);

/* Delete item(s) specified by pointer */
llist_item *llist_delete_item(llist_item *list, void *item);

/* Get list count */
int llist_count(llist_item *list);

/* Get item specified by index */
void *llist_get(llist_item *list, int index);

/* Get item index */
int llist_get_index(llist_item *list, void *item);

/* Set item specified by index */
llist_item *llist_set(llist_item *list, int index, void *item);

/* Get first item in list */
llist_item *llist_first(llist_item *list);

/* Get last item in list */
llist_item *llist_last(llist_item *list);

/* Get prevous item in list */
llist_item *llist_prev(llist_item *list);

/* Get next item in list */
llist_item *llist_next(llist_item *list);

#ifdef __cplusplus
}
#endif 

#endif /* _LLIST_H */
