#include "compile.h"
#include "progress.h"
#include "plist.h"
#include "edit.h"
#include "ui.h"
#include "conf.h"

CMotorCompiler::CMotorCompiler() {
}

CMotorCompiler::~CMotorCompiler() {
}

void CMotorCompiler::freeparseregex(void *p) {
    regex_t *r = (regex_t *) p;
    regfree(r);
    delete r;
}

bool CMotorCompiler::runmake(makenamemode mm, string target) {
    bool ret = false;
    int cpipe[2];

    string makecl = (string) MAKECMD + " -s -f " + MAKEFILE_NAME + " " + target;
    proj->chdir();
    
    nwarn = nerr = nfiles = 0;
    messages.clear();

    if(!pipe(cpipe)) {
        int makepid = fork(), status;
        
        if(!makepid) {
            dup2(cpipe[1], STDOUT_FILENO);
            dup2(cpipe[1], STDERR_FILENO);
            close(cpipe[1]); close(cpipe[0]);

            //execlp("/bin/sh", "/bin/sh", "-c", makecl.c_str());
            system(makecl.c_str());
            _exit(1);
        } else {
            close(cpipe[1]);
            if(ret = (fpipe = fdopen(cpipe[0], "r"))) ret = readpipe();
            waitpid(makepid, &status, 0);
        }
    }
    
    return ret;
}

bool CMotorCompiler::readpipe() {
    buildrule *br = 0, *nbr;
    regmatch_t rm[4];
    linkedlist parse;
    string substr, buf, pname, errstr = (string) MAKECMD + ": ***";
    bool ret = true;
    int i, mescount, l, pos;
    char sbuf[1024];

    parse.freeitem = &freeparseregex;
    CMotorProgress progress(_(" Compilation progress "));
    
    while(!feof(fpipe)) {
        freads(fpipe, sbuf, 1024);
        buf = sbuf;
        if(!buf.size()) continue;

        if(buf.substr(0, 10) == "motor: ***") {
            buf.replace(0, 11, "");
            substr = getword(buf, "|");

            if(pname != substr) {
                pname = substr;
                progress.print(true, pmheader, _(" Building ") + pname);
            }

            progress.print(true, pminfo, (string) "   " + justfname(buf.c_str()));
            nfiles++;

            if(nbr = proj->tmpl->getbuildrule(buf)) {
                for(br = nbr, parse.empty(), i = 0; i < br->parse->count; i++) {
                    regex_t *r = new regex_t;
                    const char *cmask = (((parserule *) br->parse->at(i))->mask).c_str();
                    if(!regcomp(r, cmask, REG_EXTENDED)) parse.add(r); else delete r;
                }
            }

        } else {

            bool found = false;
            compilermessage cm;
                
            for(i = 0; i < parse.count; i++)
            if(!regexec((regex_t *) parse.at(i), buf.c_str(), 4, rm, 0)) {
                parserule *pr = (parserule *) br->parse->at(i);
                        
                for(int k = 1; k <= 3; k++)
                if(rm[k].rm_so != -1) {
                    substr = buf.substr(rm[k].rm_so);
                    substr.resize(rm[k].rm_eo-rm[k].rm_so);

                    if(k-1 == pr->nfname) {
                        cm.fname = plist.get(pname)->absfname(substr);
                    } else if(k-1 == pr->ndescr) {
                        cm.text = substr;
                    } else if(k-1 == pr->nline) {
                        cm.line = atol(substr.c_str());
                    }
                }

                if(pr->iserror) nerr++; else nwarn++;
                found = true;
                break;
            }

            if(!found) cm.text = buf;
            messages.push_back(cm);
        }

	if((pos = buf.find(errstr)) != -1) {
	    buf.replace(0, pos, "");

            if(buf.substr(errstr.size()) != "Warning: ") {
                ret = false;
                break;
            }
        }
    }

    fclose(fpipe);
    return ret;
}

int CMotorCompiler::resultkeys(verticalmenu *ref, int k) {
    switch(k) {
        case KEY_F(6):
        case KEY_F(10):
            return 0;
    }

    return -1;
}

void CMotorCompiler::select(int n) {
    if(n-1 < messages.size()) {
        compilermessage &m = messages[n-1];
        if(!m.fname.empty()) ed.openfile(m.fname, m.line);
        core.bottom(m.text);
    }
}

void CMotorCompiler::show() {
    int i, h;
    vector<compilermessage>::iterator mi;

    if(!messages.empty()) {
        if((h = messages.size()) > LINES-3) h = LINES-3;
        if(current > messages.size()) current = 1;
    
        resultsm = verticalmenu(conf.getcolor(cp_menufr), conf.getcolor(cp_menusel));
        resultsm.setwindow(textwindow(0, LINES-3-h, COLS-1, LINES-2, conf.getcolor(cp_menufr)));

        for(mi = messages.begin(); mi != messages.end(); mi++)
        if(!mi->fname.empty()) {
            resultsm.additem((string) " " + justfname(mi->fname.c_str()) + ":" + mi->text + " ");
        } else {
            resultsm.additem((string) " " + mi->text);
        }

        resultsm.otherkeys = &resultkeys;
        resultsm.setpos(current-1);
    
        i = resultsm.open();
        resultsm.close();
        if(i) select(current = i);

        core.redraw();
    }
}

void CMotorCompiler::quickmove(int delta) {
    int npos = current + delta;
    
    if((npos > messages.size()) || (npos <= 0)) {
        if(npos <= 0) core.bottom(_("At the start of messages"));
        else core.bottom(_("At the end of messages"));
    } else {
        select(current = npos);
    }
}

void CMotorCompiler::output() {
}
