#include "conf.h"
#include "project.h"
#include "edit.h"
#include "plist.h"
#include "ui.h"

externalkey::externalkey(string defline) {
    string buf = getword(defline);
    keyname = buf;
    
    if((toupper(buf[1]) == 'F') && isdigit(buf[2])) {
        keycode = KEY_F(atol(buf.substr(2).c_str()));
    } else {
        keycode = buf[1];
    }

    if(buf[0] == '^') {
        keycode = CTRL(keycode);
    } else {
        keycode = ALT(keycode);
    }

    commandline = defline;
}

externalkey::~externalkey() {
}

void externalkey::tomenu(verticalmenu *m) {
    m->additemf(" %-10s %s",
        keyname.c_str(),
        comment.size() ? comment.c_str() : commandline.c_str()
    );
}
        
bool externalkey::thekey(int code) {
    return keycode == code;
}

void externalkey::execute() {
    char *p;
    int i;

    endwin();
    attrset(color(0));
    for(i = 0; i < LINES; i++, printf("\n\n"));
    fflush(stdout);

    pparamslist *params = pparamslist_create();
    proj->populateparselist(params);
    ed.editclose(CA_DONTASK | CA_DONTSHOW);

    system(p = sstrparse(commandline.c_str(), params));
    pparamslist_free(params);

    refresh();
    proj->chdir();
    ed.editload();
    ed.redraw();
    
    delete p;
}

void externalkey::setcomment(string cmnt) {
    comment = cmnt;
}

// --------------------------------------------------------------------------

externalkey *externalkeylist::get(int n) {
    return (externalkey *) at(n);
}

void externalkeylist::showlist() {
    int i;
    verticalmenu m(conf.getcolor(cp_menufr), conf.getcolor(cp_menusel));

    m.setwindow(textwindow(1, 1, (int) (COLS*0.8),
        count > LINES-4 ? LINES-4 : count+2, conf.getcolor(cp_menufr), 0,
        conf.getcolor(cp_menuhl), _(" External programs ")));

    for(i = 0; i < count; i++) get(i)->tomenu(&m);

    i = m.open();
    m.close();
    
    if(i) get(i-1)->execute();
}

void externalkeylist::execkey(int code) {
    for(int i = 0; i < count; i++) {
        externalkey *k = get(i);
        if(k->thekey(code)) k->execute();
    }
}

// --------------------------------------------------------------------------

CMotorConfig::CMotorConfig() {
    check();
    read();
    readthemes();
}

CMotorConfig::~CMotorConfig() {
}

void CMotorConfig::commandline(int argc, char *argv[]) {
    int i;
    while((i = getopt(argc, argv, "")) != EOF);
    if(argv[optind]) plist.setcur(argv[optind]);
}

int CMotorConfig::listkeys(verticalmenu *m, int k) {
    switch(k) {
        case KEY_RIGHT:
        case KEY_LEFT: conf.cbar->movebar(k); break;
        case KEY_IC: conf.cbar->item = 0; break;
        case KEY_DC: conf.cbar->item = 1; break;
    }

    switch(k) {
        case KEY_IC:
        case KEY_DC:
            conf.cbar->redraw();
            return m->getpos()+1;
    }

    return -1;
}

void CMotorConfig::check() {
    struct stat st;

    string dirname = getenv("HOME");
    dirname += "/.motor";

    if(stat(dirname.c_str(), &st)) mkdir(dirname.c_str(), S_IRWXU);

    dirname += "/projects";
    if(stat(dirname.c_str(), &st)) mkdir(dirname.c_str(), S_IRWXU);
}

void CMotorConfig::addcvs() {
    string ans = core.input("", _("cvsroot to add: "));
    if(!ans.empty()) conf.repositories.push_back(ans);
}

void CMotorConfig::gensettings(treeview *tree) {
    int oel = tree->menu.getpos(), ncvsr, ncvs;
    vector<string>::iterator si;

    tree->clear();

    ncvs  = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Version control "));
    ncvsr = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" CVS repositories "));

    tree->addleaff(ncvs, 0, 0, _(" auto \"update\" on project opening : %s "), stryesno(autocvsupdate));
    tree->addleaff(ncvs, 0, 0, _(" auto \"commit\" on project closing : %s "), stryesno(autocvscommit));

    for(si = repositories.begin(); si != repositories.end(); si++)
        tree->addleaf(ncvsr, 0, 0, (string) " " + *si + " ");

    tree->menu.setpos(oel);
}

void CMotorConfig::execute() {
    int n;
    char buf[512];

    textwindow *w = new textwindow(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT,
        conf.getcolor(cp_menufr), TW_CENTERED,
        conf.getcolor(cp_menuhl), _(" Motor settings "));
    w->open();

    treeview *tree = new treeview(w->x1+1, w->y1+1, w->x2, w->y2-3,
	conf.getcolor(cp_menu), conf.getcolor(cp_menusel),
	conf.getcolor(cp_menu), conf.getcolor(cp_menu)
    );

    horizontalbar *bar = new horizontalbar(w->x2-1, w->y2-1,
        conf.getcolor(cp_menu), conf.getcolor(cp_menusel),
        _("Add"), _("Remove"), _("Change"), 0);

    bar->align(baleft);
    bar->item = 2;
    tree->menu.otherkeys = &listkeys;

    w->separatey(w->y2-w->y1-2);
    bar->redraw();
    bool finished = false;

    while(!finished) {
        cbar = bar;
        gensettings(tree);
        tree->open(&n);
        buf[0] = 0;

        if(n) {
            switch(bar->item) {
                case 0: addcvs(); break;
                case 1: if(n >= 5) repositories.erase(repositories.begin()+n-5); break;
                case 2:
                    if(n >= 5) {
                        string ans = core.input(repositories[n-5], "cvsroot: ");
                        if(!ans.empty()) repositories[n-5] = ans;
                    } else {
                        switch(n) {
                            case 2: autocvsupdate = !autocvsupdate; break;
                            case 3: autocvscommit = !autocvscommit; break;
                        }
                    }
                    break;
            }
        } else {
            finished = true;
        }
    }

    w->close();

    delete w;
    delete bar;
    delete tree;
}

void CMotorConfig::read() {
    string sect, buf, confname = (string) getenv("HOME") + "/.motor/cvs";
    ifstream f(confname.c_str());
    
    if(f.is_open()) {
        while(getconf(sect, buf, f)) {
            if(sect == "autoupdate") autocvsupdate = (bool) atol(buf.c_str()); else
            if(sect == "autocommit") autocvscommit = (bool) atol(buf.c_str()); else
            if(sect == "repositories") repositories.push_back(buf);
        }
        f.close();
    }

    confname = (string) getenv("HOME") + "/.motor/external";
    if(access(confname.c_str(), R_OK)) confname = (string) SHAREDIR + "/motor/external";
    
    externalkey *prev = 0;

    f.open(confname.c_str());
    if(f.is_open()) {
        while(!f.eof()) {
            getline(f, buf);

            if(!buf.empty())
            if(buf[0] == '#') {
                if(prev) {
                    do buf.replace(0, 1, ""); while(buf[0] == ' ');
                    prev->setcomment(buf);
                }
                prev = 0;
            } else if(buf[0]) {
                prev = new externalkey(buf);
                extkeys.add(prev);
            }
        }
        f.close();
    }
}

void CMotorConfig::write() {
    vector<string>::iterator si;
    string fname = (string) getenv("HOME") + "/.motor/cvs";
    ofstream f(fname.c_str());

    if(f.is_open()) {
        f << "%autoupdate" << endl << (autocvsupdate ? '1' : '0') << endl;
        f << "%autocommit" << endl << (autocvscommit ? '1' : '0') << endl;

        f << "%repositories" << endl;

        for(si = repositories.begin(); si != repositories.end(); si++)
            f << *si << endl;

        f.close();
    }
}

int CMotorConfig::findcolor(string s) {
    int i;
    string colors[] = {"BLACK", "RED", "GREEN", "YELLOW", "BLUE", "MAGENTA", "CYAN", "WHITE", ""};
    
    for(i = 0; i < s.size(); i++) s[i] = toupper(s[i]);
    for(i = 0; !colors[i].empty() && (s != colors[i]); i++);
    
    return i;
}

void CMotorConfig::readthemes() {
    ifstream f;
    string buf, sub, color, sect;
    int nspos, ntpos, npos, npair, nfg, nbg, i;
    bool schemeread = false;
    
    boldcolors.clear();

    // Setting default color parameters

    init_pair(cp_input, COLOR_BLACK, COLOR_CYAN);
    init_pair(cp_bottom, COLOR_BLACK, COLOR_WHITE);
    init_pair(cp_bottomhl, COLOR_RED, COLOR_WHITE);
    init_pair(cp_menu, COLOR_BLACK, COLOR_CYAN);
    init_pair(cp_menufr, COLOR_BLACK, COLOR_CYAN);
    init_pair(cp_menusel, COLOR_WHITE, COLOR_BLACK);
    init_pair(cp_menuhl, COLOR_WHITE, COLOR_CYAN);
    init_pair(cp_debug_current, COLOR_BLACK, COLOR_CYAN);
    init_pair(cp_debug_breakpoint, COLOR_WHITE, COLOR_RED);

    // Now read the themes file

    string fname = (string) getenv("HOME") + "/.motor/themes";
    if(access(fname.c_str(), R_OK)) fname = (string) SHAREDIR + "/motor/themes";

    f.open(fname.c_str());
    if(f.is_open()) {
        while(getconf(sect, buf, f)) {
            sub = getword(sect);
            
            if(sub == "general") {
                if(buf.substr(0, 6) == "scheme") {
                    getword(buf);
                    schemename = getword(buf);
                }
            } else if((sub == "colorscheme") && (sect == schemename)) {
                schemeread = true;

                sub = getword(buf);
                color = getword(buf);

                if(i = color.find("/")) {
                    nfg = findcolor(color.substr(0, i));
                    nbg = findcolor(color.substr(i+1));
                }

                if(sub == "input") npair = cp_input; else
                if(sub == "status_norm") npair = cp_bottom; else
                if(sub == "status_hlight") npair = cp_bottomhl; else
                if(sub == "border") npair = cp_menufr; else
                if(sub == "menu_norm") npair = cp_menu; else
                if(sub == "menu_bar") npair = cp_menusel; else
                if(sub == "menu_hlight") npair = cp_menuhl; else
                if(sub == "debug_current") npair = cp_debug_current; else
                if(sub == "debug_breakp") npair = cp_debug_breakpoint;
                else continue;

                init_pair(npair, nfg, nbg);
                if(buf == "bold") boldcolors.push_back(npair);
            } else if(schemeread) {
                break;
            }
        }
        f.close();
    }
}

int CMotorConfig::getcolor(int paircode) {
    return find(boldcolors.begin(), boldcolors.end(), paircode) != boldcolors.end() ?
        boldcolor(paircode) : color(paircode);
}

string CMotorConfig::getschemename() {
    return schemename;
}

int CMotorConfig::getreposcount() {
    return repositories.size();
}

string CMotorConfig::getrepos(int n) {
    if(n < repositories.size()) return repositories[n];
    else return "";
}

// --------------------------------------------------------------------------

bool getconf(string &st, string &buf, ifstream &f) {
    bool ret = false;
    static string sect;

    while(!f.eof() && !ret) {
        getline(f, buf);

        if(buf.size())
        switch(buf[0]) {
            case '%':
                sect = buf.substr(1);
                break;
            case '#':
                break;
            default:
                ret = buf.size();
                break;
        }
    }

    st = sect;
    return ret;
}

