#include "cvs.h"
#include "ui.h"
#include "project.h"

CMotorCVS::CMotorCVS() {
    nocvs = pathfind("cvs", getenv("PATH") ? getenv("PATH") : "").empty();
}

CMotorCVS::~CMotorCVS() {
}

void CMotorCVS::setrepos(string cr) {
    cvsroot = cr;
}

void CMotorCVS::checkout(string module, string dir) {
    string cmd, localdir, tdir;

    if(!nocvs) {
        if(access(dir.c_str(), F_OK)) mkdir(dir.c_str(), S_IRWXU);
        tdir = "motor." + i2str(getpid()) + ".checkout";
        cmd = (string) "checkout -d \"" + tdir + "\" \"" + module + "\"";
        run(dir, cmd, ckcheckout);

        chdir(dir.c_str());
        cmd = "cp -a " + tdir + "/* . && rm -rf " + tdir;
        system(cmd.c_str());
    }
}

void CMotorCVS::update(string dir) {
    if(!nocvs) run(dir, "update", ckupdate);
}

void CMotorCVS::commit(string dir, string msg) {
    FILE *f;
    string fname = (string) "/tmp/cvs." + i2str(getpid()) + "." + i2str(time(0)) + ".log";

    if(!nocvs) {
        if(f = fopen(fname.c_str(), "w")) {
	    fwrite(msg.c_str(), msg.size(), 1, f);
	    fclose(f);
	    run(dir, (string) "commit -F " + fname, ckcommit);
	    unlink(fname.c_str());
        } else {
	    result = rkaborted;
        }
    }
}

void CMotorCVS::status(string dir, vector<cvsupdated> &lst) {
    if(!nocvs) {
        updlist.clear();
        run(dir, "status", ckstatus);
        lst = updlist;
    }
}

void CMotorCVS::statusv(string dir, vector<cvsrelease> &lst) {
    if(!nocvs) {
        rlist.clear();
        run(dir, "status -v", ckstatusv);
        lst = rlist;
    }
}

void CMotorCVS::add(string dir, string fname) {
    if(!nocvs) run(dir, (string) "add \"" + fname + "\"", ckadd);
}

void CMotorCVS::remove(string dir, string fname, bool physically) {
    if(!nocvs) {
        string tname;
        chdir(dir.c_str());

        if(!physically) {
            tname = fname + (string) ".kept." + i2str(getpid());
            link(fname.c_str(), tname.c_str());
        }
    
        unlink(fname.c_str());
        run(dir, (string) "remove \"" + fname + "\"", ckremove);
        if(!physically) rename(tname.c_str(), fname.c_str());
    }
}

void CMotorCVS::tag(string dir, string relname) {
    if(!nocvs) run(dir, (string) "tag \"" + relname + "\"", cktag);
}

int CMotorCVS::linkfile(const char *fname, const struct stat *sb, int flag) {
    string tdir;
    
    if(flag == FTW_F)
    if(proj->contains(fname)) {
        tdir = proj->rootdir + "/motor." + i2str(getpid()) + ".cvsimport/" + proj->relativefname(fname);
        getrword(tdir, "/");
        mksubdirs(tdir.c_str());
        tdir += (string) "/" + justfname(fname);
        link(fname, tdir.c_str());
    }

    return 0;
}

void CMotorCVS::import(string dir, string module) {
    string tdir, cmd;

    if(!nocvs) {
        ftw(dir.c_str(), &linkfile, 5);
        tdir = dir + "/motor." + i2str(getpid()) + ".cvsimport";

        cmd = "import -m \"Imported into the CVS repository\" " +
            module + " " + justfname(module.c_str()) + " start";

        run(tdir, cmd, ckimport);
        cmd = "rm -rf " + tdir;
        system(cmd.c_str());
    }
}

void CMotorCVS::getrelease(string dir, string module, string relname, string fname) {
    if(!nocvs) {
        string cmd;
        
        if(fname.empty()) {
            cmd = "diff -u -r \"";
            cmd += relname;
            cmd += "\" | grep -v \"^\\\\\\\\\" | patch -u -p0 --batch --quiet";
        } else {
            cmd = "diff -u -r \"";
            cmd += relname;
            cmd += "\" \"";
            cmd += fname;
            cmd += "\" | grep -v \"^\\\\\\\\\" | patch -u -p0 --batch --quiet";
        }

        run(dir, cmd, ckcheckoutr);
    
        if(result == rkok) {
            string forig = fname + ".orig";
            unlink(forig.c_str());
        }
    }
}

void CMotorCVS::log(string dir, string fname, vector<cvslog> &lst) {
    if(!nocvs) {
        loglist.clear();
        run(dir, "log \"" + fname + "\"", cklog);
        lst = loglist;
    }
}

void CMotorCVS::run(string indir, string cvscommand, cvscommandkind ck) {
    if(!pipe(cpipe)) {
        fpipe = fdopen(cpipe[0], "r");
        int makepid = fork(), status;
        
        if(!makepid) {
            dup2(cpipe[1], STDOUT_FILENO);
            dup2(cpipe[1], STDERR_FILENO);

            close(cpipe[1]);
            close(cpipe[0]);

	    chdir(indir.c_str());
            
            string cmd = "cvs -d " + cvsroot + " " + cvscommand + " 2>&1";
            system(cmd.c_str());
            
            _exit(1);
        } else {
            close(cpipe[1]);
            readpipe(ck);
            waitpid(makepid, &status, 0);
        }
    } else {
        throw _("runcvs: unable to create a pipe");
    }
}

void CMotorCVS::readpipe(cvscommandkind ck) {
    string buf, sub, fname, curfname, lbuf;
    cvslog *clog = 0;
    char sbuf[1024];
    bool tags = false;
    int pos;
    vector<cvsrelease>::iterator ir;
    
    result = rkok;

    while(!feof(fpipe)) {
	freads(fpipe, sbuf, 1024);
        buf = sbuf;

        if(buf.find("cannot find password") != -1) {

            result = rknotlogged;
            
        } else if((pos = buf.find(" aborted]: ")) != -1) {

            buf.insert(0, "~");
            buf.insert(buf.find("]: ")+3, "~");
            core.bottom(buf);
            if(result == rkok) result = rkaborted;

        } else if((buf.find("failed") != -1) && (buf.find("Examining") == -1)) {

            core.bottom(buf);
            if(result == rkok) result = rkaborted;
            break;
            
        } else if(ck == ckstatus) {
            sub = getword(buf);
            
            if(sub == "File:") {
                fname = trailcut(getword(buf, "\t"));
                sub = getword(buf);
                
                if(sub == "Status:") {
                    cvsmodificationkind k = mknone;

                    if(buf == "Needs Patch") k = mkremote; else
                    if(buf == "Locally Modified") k = mklocal; else
                    if(buf == "Locally Added") k = mklocaladded; else
                    if(buf == "Locally Removed") k = mklocalremoved;

                    if(k != mknone) {
                        if(fname.substr(0, 8) == "no file ") fname.replace(0, 8, "");
                        updlist.push_back(cvsupdated(fname, k));
                    }
                }
            }
        } else if(ck == ckstatusv) {
            sub = getword(buf);

            if(sub == "File:") {
                curfname = trailcut(getword(buf, "\t"));
                sub = getword(buf);
                tags = false;

                if(sub == "Status:")
                if(curfname.substr(0, 8) == "no file ")
                    curfname.replace(0, 8, "");
                    
            } else if(sub == "Existing") {

                if((sub = getword(buf)) == "Tags:")
                if(!curfname.empty()) tags = true;
                
            } else if(tags = !buf.empty()) {
                if((ir = find(rlist.begin(), rlist.end(), sub)) == rlist.end()) {
                    rlist.push_back(cvsrelease(sub));
                    ir = rlist.end()-1;
                }

                ir->files.push_back(curfname);
            }
            
        } else if(ck == cklog) {
            lbuf = buf;
            sub = getword(buf);

            if(sub == "revision") {

                clog = new cvslog;
                clog->revision = buf;
                
            } else if((sub == "date:") && clog) {

                clog->date = getword(buf, ";");
                if(getword(buf) == "author:") clog->author = getword(buf, ";");
                
            } else if(clog) {

                if((lbuf.substr(0, 10) == sub.assign(10, '-')) ||
                (lbuf.substr(0, 10) == sub.assign(10, '='))) {
                    if(clog) {
                        loglist.push_back(*clog);
                        delete clog;
                        clog = 0;
                    }
                } else {
                    if(!clog->log.empty()) clog->log += "\n";
                    clog->log += lbuf;
                }
            }
        }
    }

    fclose(fpipe);
}

// --------------------------------------------------------------------------

cvsupdated::cvsupdated(const string afname = "", const cvsmodificationkind akind = mknone) {
    fname = afname;
    kind = akind;
}

cvsupdated::~cvsupdated() {
}

bool cvsupdated::operator == (const cvsmodificationkind &akind) const {
    return kind == akind;
}

bool cvsupdated::operator != (const cvsmodificationkind &akind) const {
    return kind != akind;
}

// --------------------------------------------------------------------------

cvsrelease::cvsrelease(const string aname = "") {
    name = aname;
}

cvsrelease::~cvsrelease() {
}

bool cvsrelease::operator == (const string &aname) const {
    return name == aname;
}

bool cvsrelease::operator != (const string &aname) const {
    return name != aname;
}

