#include "debugp.h"
#include "debug.h"

debugpoint::debugpoint(dpkind k, string text) {
    int npos;
    
    switch(kind = k) {
        case dpwatch:
            watch.name = text;
            break;
        case dpbreakpoint:
            if((npos = text.find("\t")) != -1) {
                breakp.fname = text;
                breakp.fname.resize(npos);
                breakp.line = atol(text.substr(npos+1).c_str());
                breakp.permanent = true;
            }
            break;
    }
}

debugpoint::~debugpoint() {
}

void debugpoint::todebugger() {
    switch(kind) {
        case dpwatch:
            deb.watches.push_back(watchexpr(watch.name));
            break;
        case dpbreakpoint:
            deb.addbreakpoint(breakpoint(breakp.fname, breakp.line, breakp.permanent));
            break;
    }
}

dpkind debugpoint::getkind() {
    return kind;
}

watchexpr debugpoint::getwatch() {
    return watch;
}

breakpoint debugpoint::getbreakpoint() {
    return breakp;
}

// --------------------------------------------------------------------------

progstackitem::progstackitem(string line) {
    nextline(line);
}

progstackitem::~progstackitem() {
}

void progstackitem::nextline(string line) {
    int fpos, fsize;   // "at/from" position and length
    int lpos;          // line number position
    string aword;
    
    if(line[0] == '#') {
        if((fpos = line.find(" in ")) != -1) {
            line.replace(0, fpos+4, "");
        } else {
            getword(line);
        }

        funcname = getword(line);
        nextline(line);
    } else {
        leadcut(line);

        if((fpos = line.find(" at ")) != -1) {
            fsize = 4;
        } else {
            fpos = line.find(" from ");
            fsize = 6;
        }

        if(fpos != -1) {
            fname = line.substr(fpos+fsize);

            if((lpos = fname.rfind(":")) != -1) {
                lineno = strtol(fname.c_str()+lpos+1, 0, 0);
                fname.resize(lpos);
            }
            
            line.resize(fpos);
        }

        funcargs += leadcut(line);
    }
}

string progstackitem::getfname() const {
    return justfname(fname.c_str());
}

string progstackitem::getfunction() const {
    return funcname;
}

string progstackitem::getargs() const {
    return funcargs;
}

int progstackitem::getline() const {
    return lineno;
}

// --------------------------------------------------------------------------

breakpoint::breakpoint() {
}

breakpoint::breakpoint(const string afname, int aline, bool apermanent) {
    fname = afname;
    line = aline;
    permanent = apermanent;
}

breakpoint::~breakpoint() {
}

bool breakpoint::operator == (const breakpoint &abp) const {
    string tfname = justfname(fname.c_str());
    string cfname = justfname(abp.fname.c_str());

    return (line == abp.line) &&
        (permanent == abp.permanent) &&
        (tfname == cfname);
}

bool breakpoint::operator == (const int &aid) const {
    return id == aid;
}

bool breakpoint::operator == (const bool &aperm) const {
    return permanent == aperm;
}

bool breakpoint::operator != (const breakpoint &abp) const {
    return !(*this == abp);
}

bool breakpoint::operator != (const int &aid) const {
    return !(*this == aid);
}

bool breakpoint::operator != (const bool &aperm) const {
    return !(*this == aperm);
}

// --------------------------------------------------------------------------

watchexpr::watchexpr() {
}

watchexpr::watchexpr(const string aname, const string aval) {
    name = aname;
    val = aval;
}

watchexpr::~watchexpr() {
}

