#include "dist.h"
#include "ui.h"
#include "project.h"
#include "conf.h"
#include "edit.h"

CMotorDist::CMotorDist() {
}

CMotorDist::~CMotorDist() {
}

void CMotorDist::editline(string *param, string prompt, bool dir) {
    string ans;

    if(dir) ans = core.inputdir(*param, prompt);
    else ans = core.input(*param, prompt);
    
    if(!ans.empty()) *param = ans;
}

void CMotorDist::addinstrule() {
    string fname, dest;
    core.fm->chroot = true;
    fname = core.inputfile(fname, _("file name to install: "));
    core.fm->chroot = false;
    
    if(!fname.empty()) {
        dest = core.inputdir(dest, _("destination directory: "));
        if(!dest.empty()) proj->addinst(proj->relativefname(fname), dest);
    }
}

void CMotorDist::dialog() {
    char *p;
    int n, i, destno = 0;
    distmethod *dst;
    installrule *ir;

    if(!proj->tmpl->getdistcount()) {
        char buf[512];
        sprintf(buf, _("No distribution methods defined for template %s"), proj->tmpl->getname().c_str());
        core.bottom(buf);
        return;
    }

    textwindow w(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT, conf.getcolor(cp_menufr),
        TW_CENTERED, conf.getcolor(cp_menuhl),
        _(" Output distribution options "));

    w.open();

    treeview *tree = new treeview(w.x1+1, w.y1+1, w.x2, w.y2-2,
    conf.getcolor(cp_menu), conf.getcolor(cp_menusel), conf.getcolor(cp_menu), conf.getcolor(cp_menu));

    bar = new horizontalbar(w.x2-1, w.y2-1, conf.getcolor(cp_menu),
    conf.getcolor(cp_menusel), _("Tag release"), _("Add"), _("Remove"),
    _("Change"), _("Create"), 0);

    w.separatey(w.y2-w.y1-2);

    bar->align(baleft);
    bar->item = 3;
    bar->redraw();
    tree->menu.otherkeys = &dialogkeys;
    core.fm->setstartpoint(proj->rootdir.c_str());

    bool finished = false, proceed;

    while(!finished) {
        tree->clear();
        dst = proj->tmpl->getdist(destno);

        int npackage = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Package "));
        int noptions = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Options "));
        int ninst = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Install rules "));

        tree->addleaff(npackage, 0, 0, _(" Type : %s "), dst->getname().c_str());
        tree->addleaff(npackage, 0, 0, _(" Version : %s "), proj->version.c_str());
        tree->addleaff(npackage, 0, 0, _(" Output directory : %s "), proj->packgendir.c_str());

        tree->addleaff(noptions, 0, 0, _(" Use autoconf : %s "), stryesno(proj->pautoconf));
        tree->addleaff(noptions, 0, 0, _(" Add connected libraries : %s "), stryesno(proj->pconnected));
        tree->addleaff(noptions, 0, 0, _(" Include motor project info : %s "), stryesno(proj->paddinfo));
        tree->addleaff(noptions, 0, 0, _(" Additional install command : %s "), proj->packinstcmd.c_str());

        for(i = 0; i < proj->getinstcount(); i++)
            tree->addleaff(ninst, 0, 0, " %s -> %s ",
            proj->getinst(i)->getfname().c_str(),
            proj->getinst(i)->getdest().c_str());

        tree->open(&n);

        if(n)
        switch(bar->item) {
            case 0:
		if(proj->cvsenabled()) core.cvscommand(cktag);
                else core.bottom(_("This project isn't under the CVS control"));
		break;
            case 1: addinstrule(); break;
            case 2: if(n >= 11) proj->removeinst(n-11); break;
            case 3:
                switch(n) {
                    case 2: if(++destno >= proj->tmpl->getdistcount()) destno = 0; break;
                    case 3: editline(&proj->version, _("package version: "), false); break;
                    case 4: editline(&proj->packgendir, _("output directory to put the package to: "), true); break;
                    case 6: proj->pautoconf = !proj->pautoconf; break;
                    case 7: proj->pconnected = !proj->pconnected; break;
                    case 8: proj->paddinfo = !proj->paddinfo; break;
                    case 9: editline(&proj->packinstcmd, _("command line to execute after installing the files: "), false); break;
                }
                break;

            case 4:
                proceed = true;
                core.bottom(_("Generating a package"));
                if(proj->pautoconf) proceed = autoconf();
                if(proceed) {
		    ed.saveall(false);
                    dst->write();
                    dst->generate();
                }
		finished = true;
                break;
        } else {
            finished = true;
        }
    }

    w.close();
    
    delete bar;
    delete tree;
}

int CMotorDist::dialogkeys(verticalmenu *m, int k) {
    switch(k) {
        case KEY_RIGHT:
        case KEY_LEFT: dist.bar->movebar(k); break;
        case KEY_IC: dist.bar->item = 1; break;
        case KEY_DC: dist.bar->item = 2; break;
    }

    switch(k) {
        case KEY_IC:
        case KEY_DC:
            dist.bar->redraw();
            return m->getpos()+1;
    }
    
    return -1;
}

void CMotorDist::brushautoconf() {
    FILE *f;
    char buf[512];
    int i;
    
    if(f = fopen("configure.in", "r")) {
        FILE *of = fopen("configure.in.temp.motor", "w");

        while(!feof(f)) {
            freads(f, buf, 512);
            if(!strncmp(buf, "AC_OUTPUT(", 10)) {
                fprintf(of, "AC_OUTPUT(Makefile");

                if(proj->pconnected)
                for(i = 0; i < proj->getfilecount(); i++) {
                    projectfile *pf = proj->getfile(i);
                    if(pf->getfolder()->getelem() == feproj) {
                        fprintf(of, " %s/Makefile", pf->getfname().c_str());
                    }
                }
                
                fprintf(of, ")\n");
            } else {
                if(strcmp(buf, "AC_PROG_INSTALL")) fprintf(of, "%s\n", buf);
            }
        }
        
        fclose(f);
        fclose(of);
        rename("configure.in.temp.motor", "configure.in");
    }
}

bool CMotorDist::autoconf() {
    char *fname = "configure.in";
    bool proceed = true;
    int i;

    proj->chdir();

    if(access(fname, F_OK)) {
        core.bottom(_("autoconf script not found, executing autoscan .."));

        if(!fork()) {
            execlp("autoscan", "autoscan", 0);
            _exit(1);
        } else {
            wait(&i);
        }
        
        rename("configure.scan", "configure.in");
        brushautoconf();

        proj->addfile("configure.in");
        cvs.add(proj->rootdir, "configure.in");

        ed.openfile("configure.in", 0, true);
        core.bottom(_("Edit your configure.in and open the distribution dialog again when ready"));
        proceed = false;
    }

    return proceed;
}

