#include "edit.h"
#include "ui.h"
#include "project.h"
#include "plist.h"
#include "conf.h"
#include "debug.h"
#include "compile.h"
#include "manager.h"
#include "grep.h"
#include "tagbrowser.h"

motoreditor::motoreditor() {
}

motoreditor::~motoreditor() {
    unlink(clipfname().c_str());
}

void motoreditor::init() {
    readsyntax();
    resize(elnormal);
    e.otherkeys = &editkeys;
    e.idle = &editidle;
}

void motoreditor::saveas() {
    string nfname = core.inputfile("", _("save as: "));

    if(!nfname.empty()) {
        free(e.getfid());
        e.setfid(strdup(nfname.c_str()));
        save();
    }
}

void motoreditor::save() {
    char *fname = e.getfid();
    FILE *f = fopen(fname, "w");

    if(f) {
        core.bottomf(_("Saving %s .."), justfname(fname));
        e.save(f, "\n");
        fclose(f);
        core.bottomf(_("File %s saved successfully"), fname);
    } else {
        core.bottomf(_("Unable to save the file %s"), fname);
    }
}

void *motoreditor::guessfname(void *ppf, void *sfname) {
    projectfile *pf = (projectfile *) ppf;
    string fname = *(string *) sfname, pfname = pf->getfname();
    motorproject *tpr;
    string *sp;
    int pos;
    void *ret = 0;

    switch(pf->getfolder()->getelem()) {
        case fefile:
            if((pos = pfname.size()-fname.size()) >= 0)
            if(!pfname.compare(fname, pos)) {
                sp = new string;
                *sp = pfname;
                ret = sp;
            }
            break;
        case feproj:
            if(tpr = plist.get(pfname)) {
                if(ret = tpr->foreachfile(&guessfname, sfname)) {
                    sp = (string *) ret;
                    *sp = tpr->absfname(*sp);
                }
            }
            break;
    }

    return ret;
}

bool motoreditor::openfile(string fname, int posline = 0, bool create = false) {
    int i, k, inode, pos;
    bool found, ret = true;
    struct stat st;
    string *pfname, tfname;

    if(!fname.empty())
    if(fname[0] != '/') {
        tfname = fname;
        pfname = (string *) proj->foreachfile(&guessfname, &tfname);

        if(!pfname) {
            tfname = justfname(fname.c_str());
            pfname = (string *) proj->foreachfile(&guessfname, &tfname);
        }

        if(pfname) {
            fname = *pfname;
            delete pfname;
        }
    }

    if(!stat(fname.c_str(), &st)) {
        inode = st.st_ino;

        // Check if the file is open already

        for(i = 0, found = false; i < e.getfcount(); i++)
        if(!stat(e.getfid(i), &st))
        if(inode == st.st_ino) {
            found = true;
            break;
        }

        // Switch to an appropriate window or load it

        if(found) {
            e.setfnum(i);
        } else {
            FILE *f = fopen(fname.c_str(), "r");

            if(f) {
                e.load(f, strdup(fname.c_str()));
                fclose(f);
            } else {
                ret = false;
            }
        }

        status();
    } else if(create) {
    
        // Check if the file is open already

        for(i = 0, found = false; i < e.getfcount(); i++)
        if((string) e.getfid(i) == fname) {
            found = true;
            break;
        }

        // Switch to an appropriate window or load it

        if(found) {
            e.setfnum(i);
        } else {
            core.bottomf(_("File %s not found"), fname.c_str());
            e.load("", strdup(fname.c_str()));
        }
    } else {
        ret = false;
    }

    if(ret) {
        e.setcolorscheme(getsyntax(fname.c_str()));
        markbreakpoints();
        if(posline) e.setpos(0, posline-1);
    } else {
//        core.bottomf(_("Unable to load the file %s"), fname.c_str());
    }

    return ret;
}

void motoreditor::load() {
    string fname = core.inputfile("", _("load file: "));
    if(!fname.empty()) openfile(fname.c_str(), 0, true);
}

void motoreditor::saveall(bool askf = false) {
    int i, cur = e.getfnum();
    
    for(i = 0; i < e.getfcount(); i++) {
        e.setfnum(i);
	if(e.modified) save();
    }

    e.setfnum(cur);
}

void motoreditor::editidle(texteditor *ref) {
    static int x = -1, y = -1;
    static bool imode, bmode;
    int nx, ny;

    ed.e.getpos(&nx, &ny);

    if((nx != x) || (ny != y) || !ref ||
    (ed.e.insertmode != imode) ||
    (ed.e.ismark() != bmode)) {
        ed.status(x = nx, y = ny, imode = ed.e.insertmode, bmode = ed.e.ismark());
    } else {
	dataready(0, 1);
    }
}

void motoreditor::switchwindow(int delta) {
    int next = e.getfnum() + delta;

    if(next >= e.getfcount()) next = 0; else
    if(next < 0) next = e.getfcount()-1;

    e.setfnum(next);
    status();
}

void motoreditor::editload() {
    FILE *ef;
    string fn, info;
    int cfn, i, k, edfn, pline, pcol;
    
    pline = pcol = edfn = 0;
    proj->chdir();
    
    for(i = 0; i < proj->getedcount(); i++) {
        fn = proj->edget(i);
        
        if(k = fn.find("\t")) {
            info = fn;
            fn.resize(k);
            info.replace(0, k+1, "");
        }

	if((ef = fopen(fn.c_str(), "r"))) {
            pcol = atol(info.c_str());

            if((k = info.find("\t"))) pline = atol(info.c_str()+k);
            if((info[info.size()-1] == '!')) cfn = edfn;
            
	    e.load(ef, strdup(fn.c_str()));
            e.setcolorscheme(getsyntax(fn));
            e.setpos(pcol, pline);
	    fclose(ef);
	    edfn++;
        }
    }

    e.setfnum(cfn);
    e.active = e.getfcount();
    markbreakpoints();
}

bool motoreditor::editclose(int options) {
    if(proj) proj->edclear();

    while(e.getfcount())
    if(!closewindow(options)) return false;

    return true;
}

bool motoreditor::closewindow(int options) {
    if(!e.getfcount()) return false;

    if(e.modified) {
        if(!(options & CA_DONTSHOW)) e.draw();
        
        if(!(options & CA_DONTASK)) {
            switch(core.askf("YNC", _("file %s was modified. save changes?"), justfname(e.getfid()))) {
                case ANSWER_YES: save(); break;
                case ANSWER_CANCEL: return false;
            }
        } else {
            save();
        }
    }

    char *fid = e.getfid();
    string p;
    int pcol, pline;

    e.getpos(&pcol, &pline);
    p = (string) fid + "\t" + i2str(pcol) + "\t" + i2str(pline);
    if(!proj->getedcount()) p += "\t!";
    proj->edadd(strdup(p.c_str()));

    e.close();

    if(!(options & CA_DONTSHOW)) {
        e.draw();
        status();
    }

    return true;
}

bool motoreditor::modified() {
    bool ret = false;
    int f = e.getfnum();
    
    for(int i = 0; i < e.getfcount(); i++) {
        e.setfnum(i);
        if(e.modified) {
            ret = true;
            break;
        }
    }

    e.setfnum(f);
    return ret;
}

void motoreditor::findtext(bool next) {
    int col, line;
    
    if(!next || findsub.empty()) {
        if(!atcursor().empty()) findsub = atcursor();
        findsub = core.input(findsub, _("find text: "));
    }
    
    if(!findsub.empty()) {
	if(e.find(findsub.c_str(), "cs", &col, &line)) {
            e.setpos(col, line);
        } else {
            core.bottom(_("Substring not found"));
        }
    }
}

void motoreditor::replace() {
    static string replsub;
    string np;
    char *p;
    int col, line, proceed;
    bool finished, replall;

    findsub = core.input(findsub, _("search substring: "));
    finished = replall = false;
    
    if(!findsub.empty()) {
        replsub = core.input(replsub, _("replacement: "));

        while(!finished && e.find(findsub.c_str(), "cs", &col, &line)) {
            e.setpos(col, line);
            
            if(!replall) {
                e.startmark();
                e.setpos(col+findsub.size(), line);
                e.endmark();
                e.redraw();
            }

            proceed = replall ? ANSWER_YES :
                core.ask("YNAC", _("replace this occurence?"));
            
            switch(proceed) {
                case ANSWER_ALL:
                    replall = true;
                case ANSWER_YES:
		    p = e.getline(line);
                    np.assign(p, col);
                    np += replsub;
                    np += p+col+findsub.size();
		    e.putline(line, strdup(np.c_str()));
		    break;
                case ANSWER_CANCEL:
                    finished = true;
		    core.bottom(_("Cancelled"));
		    break;
            }
        }
        e.clearmark();
    }
}

void motoreditor::gotoline() {
    static string nline;
    nline = core.input(nline, _("go to line: "));
    if(!nline.empty()) e.setpos(0, atol(nline.c_str()));
}

void motoreditor::editoperation(teditoperation ed) {
    FILE *f;

    switch(ed) {
        case edcut:
            if(f = fopen(clipfname().c_str(), "w")) {
	        e.copymark(f);
                e.delmark();
                fclose(f);
            }
            break;
        case edcopy:
            if(f = fopen(clipfname().c_str(), "w")) {
	        e.copymark(f);
		e.endmark();
                fclose(f);
            }
            break;
        case edpaste:
            if(f = fopen(clipfname().c_str(), "r")) {
                e.insert(f);
                e.clearmark();
                fclose(f);
            }
            break;
    }
}

void motoreditor::redraw() {
    e.draw();
    refresh();
}

void motoreditor::readsyntax() {
    ifstream f;
    string buf, sect, sub;
    int i, fg, bg, pairseq = 1;
    bool newlang = true;
    syntaxhl *sh = 0;
    
    string fname = (string) getenv("HOME") + "/.motor/syntax";
    if(access(fname.c_str(), R_OK)) fname = (string) SHAREDIR + "/motor/syntax";

    f.open(fname.c_str());
    if(f.is_open()) {
        vector<confcolorpair> pairs;
        vector<confcolorpair>::iterator pi;
        confcolorpair cp;
        
        while(getconf(sect, buf, f)) {
	    if((sect == "pairs") || (sect == ("pairs " + conf.getschemename()))) {
                if(pairseq <= 32) {
                    sub = getword(buf);
                    cp.id = atol(sub.c_str());

                    sub = getword(buf);
                    if(i = sub.find("/")) {
                        fg = conf.findcolor(sub.substr(0, i));
                        bg = conf.findcolor(sub.substr(i+1));
                    }
                
                    cp.bold = buf == "bold";
                    init_pair(cp.pair = pairseq, fg, bg);

                    pairs.push_back(cp);
                    pairseq++;
                }
	    } else if(sect == "lang") {
                sub = getword(buf);

                if(sub == "tag") {
                    if(sh) {
                        syntax.add(sh);
                        sh = 0;
                    }
                    sh = new syntaxhl;
                    sh->tag = getword(buf);
                    newlang = true;
                } else if(sub == "mask") {
                    sh->mask = getword(buf);
                } else if(sub == "all") {
                    int n, b;
                    sub = getword(buf); n = atol(sub.c_str());
                    sub = getword(buf); b = atol(sub.c_str());
                    sh->scheme = e.addscheme(n, b, false, 0);

                    if(color(cp_debug_current) != conf.getcolor(cp_debug_current))
                        e.addcolordif(sh->scheme, cp_debug_current);

                    if(color(cp_debug_breakpoint) != conf.getcolor(cp_debug_breakpoint))
                        e.addcolordif(sh->scheme, cp_debug_breakpoint);
                        
                    newlang = false;

                    for(pi = pairs.begin(); pi != pairs.end(); pi++)
                    if(pi->bold) e.addcolordif(sh->scheme, pi->pair);

                } else if(sh) {
                    hl_kind h;

                    if(sub == "alone") h = h_alone; else
                    if(sub == "eol") h = h_eol; else
                    if(sub == "symbol") h = h_symbol; else
                    if(sub == "quotes") h = h_quotes; else
                    if(sub == "block") h = h_block; else
                    if(sub == "comment") h = h_comment; else continue;

                    sub = getrword(buf);
                    e.addhighlight(sh->scheme, buf.c_str(), atol(sub.c_str()), h);
                }
	    }
	}

        if(sh) syntax.add(sh);
	f.close();
    } else {
        sh = new syntaxhl;
        memset(sh, 0, sizeof(sh));
        init_pair(1, COLOR_WHITE, COLOR_BLUE);
        sh->scheme = e.addscheme(1, 1, false, 0);
        syntax.add(sh);
    }
}

int motoreditor::getsyntax(string fname) {
    int i, r = -1, defr;
    regex_t rt;
    
    for(i = 0; i < syntax.count; i++) {
        syntaxhl *p = (syntaxhl *) syntax.at(i);
        
        if(!p->mask.empty()) {
            if(!regcomp(&rt, p->mask.c_str(), REG_EXTENDED)) {
                if(!regexec(&rt, fname.c_str(), 0, 0, 0)) r = p->scheme;
                regfree(&rt);
            }
            if(r != -1) break;
        } else {
            defr = p->scheme;
        }
    }

    return r == -1 ? defr : r;
}

void motoreditor::switchmark() {
    e.switchmark();
}

void motoreditor::switchins() {
    e.insertmode = !e.insertmode;
}

void motoreditor::shift(bool dir) {
    e.shiftident(dir ? 4 : -4);
    e.redraw();
}

void motoreditor::updatecursor() {
    e.updatecursor();
}

void motoreditor::undo() {
    e.undo();
}

void motoreditor::status() {
    verticalmenu *m = core.getpulldown(5);
    int i;
    
    m->clear();

    m->additem(_(" Next                 ^N"));
    m->additem(_(" Previous             ^P"));
    m->additem(_(" Window list          F5"));
    m->addline();

    for(i = 0; i < e.getfcount(); i++) {
        m->additemf(" %s", justfname(e.getfid(i)));
        if(i == e.getfnum()) m->setpos(i+4);
    }

    editidle(0);
    e.updatecursor();
}

void motoreditor::status(int x, int y, bool imode, bool bmode) {
    const char *p;
    string sline, pname, fname;
    char sstat[20], buf[COLS+20];

    if(p = justfname(e.getfid())) {

	attrset(conf.getcolor(cp_bottom));
        mvhline(0, 0, ' ', COLS);

        pname = _("PROJECT: ");
        pname += proj->name;

        fname = _("FILE: ");
        fname += p;

        if(fname.length() > COLS*0.31) {
            fname.resize((int) (COLS*0.27));
            fname += "..";
        }

        fname += "   ";
        int pmaxl = COLS-strlen(VERSION)-fname.length()-28;
        
        if(pname.length() > pmaxl-5) {
            pname.resize(pmaxl-5);
            pname += "..";
        }

        pname.resize(pmaxl, ' ');

        mvprintw(0, 0, "  MOTOR %s   %s%s", VERSION, fname.c_str(), pname.c_str());
        mvprintw(0, COLS-18, "[%c%c%c] %5d:%5d", imode ? '-' : 'O', bmode ? 'B' : '-', e.modified ? 'M' : '-', x, y);
	e.updatecursor();
    }
}

int motoreditor::open() {
    int ret = e.open(), i;

    if(ret == TEXTEDITOR_NOFILES) {
        attrset(color(0));
        for(i = e.y1; i < e.y2; i++) mvhline(i, 0, ' ', COLS);
    }

    return ret;
}

string motoreditor::getcurrentfname() {
    return e.getfid() ? e.getfid() : "";
}

void motoreditor::enlight(int fnum, int line, int clr) {
    e.highlight(fnum, line, clr);
}

void motoreditor::unlight(int fn, int prevline) {
    if(prevline > 0) e.unlight(fn, prevline);
    
    if(e.getfid(fn))
    if(find(deb.bpoints.begin(), deb.bpoints.end(),
    breakpoint(e.getfid(fn), prevline)) != deb.bpoints.end())
        e.highlight(fn, prevline, cp_debug_breakpoint);
}

int motoreditor::getcurrentfnum() {
    return e.getfnum();
}

void motoreditor::getpos(int *col, int *line) {
    e.getpos(col, line);
}

int motoreditor::getpos() {
    int line;
    getpos(0, &line);
    return line;
}

void motoreditor::markbreakpoints() {
    int i;
    string fname;
    vector<breakpoint>::iterator ib;

    for(i = 0; i < e.getfcount(); i++) {
        fname = justfname(e.getfid(i));
        e.clearlight(i);

        for(ib = deb.bpoints.begin(); ib != deb.bpoints.end(); ib++)
        if(ib->permanent && !ib->fname.empty())
        if(fname == justfname(ib->fname.c_str()))
            enlight(i, ib->line, cp_debug_breakpoint);

        if(deb.running)
        if(fname == justfname(core.curdebfname.c_str()))
            enlight(i, core.curdebline, cp_debug_current);
    }
}

void motoreditor::resize(edlayout l) {
    switch(l) {
        case elwatch: e.setcoords(0, 1, COLS, LINES-(int) (LINES*0.3)); break;
        case elnormal: e.setcoords(0, 1, COLS, LINES-1); break;
    }
    
    redraw();
}

void motoreditor::windowlist() {
    int i, b;
    char *p;
    dialogbox db;
    
    db.setwindow(new textwindow(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT,
        conf.getcolor(cp_menufr), TW_CENTERED,
        conf.getcolor(cp_menuhl), _(" Window list ")));

    db.setbar(new horizontalbar(conf.getcolor(cp_menu),
	conf.getcolor(cp_menusel), _("Open"), _("Close"), _("Switch to"), 0));

    db.setmenu(new verticalmenu(conf.getcolor(cp_menu), conf.getcolor(cp_menusel)));
    db.getbar()->item = 2;
    db.getmenu()->setpos(ed.getcurrentfnum());
    db.addkey(KEY_IC, 0);
    db.addkey(KEY_DC, 1);
    
    bool finished = false;

    while(!finished) {
        db.getmenu()->clear();

        for(i = 0; i < e.getfcount(); i++) {
	    db.getmenu()->additemf(0, e.getfid(i), " %s", proj->relativefname(e.getfid(i)).c_str());
	}

	if(!(finished = !db.open(i, b)))
        switch(b) {
            case 0: load(); break;
            case 1:
                e.setfnum(i-1);
                closewindow(CA_DONTSHOW);
                break;
            case 2:
                if(p = (char *) db.getmenu()->getref(i-1))
                    openfile(p, 0, false);
                finished = true;
                break;
        }
    }
    
    db.close();
}

string motoreditor::clipfname() {
    return "/tmp/konst.editor." + i2str(getpid()) + ".clip";
}

#define atcuraccept(c) (c && (isalnum(c) || strchr("_", c)))

string motoreditor::atcursor() {
    int row, col, i;
    const char *p, *st;
    string ret;
    
    e.getpos(&col, &row);

    if(p = e.getline(row))
    if(atcuraccept(p[col])) {
        for(st = p+col; (st != p) && atcuraccept(*(st-1)); st--);
        for(ret = st, i = 0; (i < ret.size()) && atcuraccept(ret[i]); i++);
        ret.resize(i);
    }

    return ret;
}

// --------------------------------------------------------------------------

int motoreditor::editkeys(texteditor *ref, int k) {
    int modifiers = getctrlkeys();

    switch(k) {
        case KEY_F( 1): core.help(); break;
        case KEY_F( 2): ed.save(); break;
	case KEY_F( 3): ref->switchmark(); break;
	case KEY_F( 4): core.debugtocursor(); break;
        case KEY_F( 5): ed.windowlist(); break;
        case KEY_F( 6): cmp.show(); break;
        case KEY_F( 7): core.debugproceed(true); break;
        case KEY_F( 8): core.debugproceed(false); break;

        case KEY_F( 9): core.build(); break;
        case KEY_F(19): core.makeclean(); break;
            
        case KEY_F(10): return core.execmenu(ref);
        case KEY_F(11):
            if(modifiers & SHIFT_PRESSED) mgr.settings(); else
            if(modifiers & CONTROL_PRESSED) mgr.dirs(); else
            mgr.files();
            break;

        case KEY_F(12):
            if(modifiers & SHIFT_PRESSED) core.setargs();
            else if(modifiers & CONTROL_PRESSED) core.reset();
            else core.run();
            break;

        case KEY_DC:
            if((modifiers & CONTROL_PRESSED) || (modifiers & SHIFT_PRESSED))
            ed.editoperation(edcut);
            break;

        case KEY_IC:
            if(modifiers & SHIFT_PRESSED) ed.editoperation(edpaste);
            else if(modifiers & CONTROL_PRESSED) ed.editoperation(edcopy);
            break;

        case CTRL('d'): ed.closewindow(0); break;
        case CTRL('f'): ed.findtext(false); break;
        case CTRL('g'): ed.findtext(true); break;
        case CTRL('r'): ed.replace(); break;
        case CTRL('e'): cmp.quickmove(+1); break;
        case CTRL('n'): ed.switchwindow(+1); break;
        case CTRL('p'): ed.switchwindow(-1); break;
        case CTRL('o'): ed.load(); break;
        case CTRL('b'): core.setbreakpoint(); break;
        case CTRL('w'): core.addwatch(ed.atcursor()); break;
        case CTRL('u'): ed.undo(); break;

        case CTRL('x'): return -1; break;

        case ALT('x'): ed.editoperation(edcut); break;
        case ALT('c'): ed.editoperation(edcopy); break;
        case ALT('v'): ed.editoperation(edpaste); break;
        case ALT('b'): core.openbreakpoints(); break;
        case ALT('w'): core.openwatch(true); break;
        case ALT('l'): ed.gotoline(); break;
        case ALT('e'): core.evaluate(ed.atcursor()); break;
        case ALT('f'): grep.exec(); break;
        case ALT('g'): grep.nextresult(); break;
        case ALT('?'): browser.exec(ed.atcursor()); break;
        case ALT('s'): core.callstack(); break;
        
        case ALT(KEY_BACKSPACE): ed.shift(SD_BACKWARD); break;
        case ALT(KEY_TAB): ed.shift(SD_FORWARD); break;
        default: conf.extkeys.execkey(k); break;
    }

    ref->updatecursor();
    return 0;
}

