#include "grep.h"
#include "conf.h"
#include "edit.h"
#include "plist.h"
#include "ui.h"

motorgrepper::motorgrepper(): bar(0), initial(true),
casesens(false), mode(gmcurrent), listpos(0), mmode(masub) {
}

motorgrepper::~motorgrepper() {
}

const char *motorgrepper::strgrepmode(grepmode m) {
    switch(m) {
        case gmcurrent: return _("current file");
        case gmfolders: return _("selected folders");
        case gmall: return _("all the project files");
        case gmconnected: return _("connected projects");
    }
}

const char *motorgrepper::strmaskmode(maskmode m) {
    switch(m) {
        case maregex: return _("regular expression");
        case masub: return _("substring");
    }
}

void motorgrepper::initbar() {
    if(bar) delete bar;
    w->redraw();
    w->separatey(w->y2-w->y1-2);
    
    if(!initial) {
        bar = new horizontalbar(w->x2-1, w->y2-1,
            conf.getcolor(cp_menu),
            conf.getcolor(cp_menusel), _("Switch to"), _("New search"), 0);
    } else {
        bar = new horizontalbar(w->x2-1, w->y2-1,
            conf.getcolor(cp_menu),
            conf.getcolor(cp_menusel), _("Change/Mark"), _("Grep"), 0);
    }

    bar->align(baleft);
    bar->item = 0;
    bar->redraw();
}

void motorgrepper::showprogress(string fname) {
    attrset(conf.getcolor(cp_menu));
    mvhline(w->y2-1, w->x1+1, ' ', w->x2-w->x1-1);
    w->writef(2, w->y2-w->y1-1, _("Wait .. %s"), fname.c_str());
}

bool motorgrepper::grepfile(string fname) {
    FILE *f;
    unsigned int l;
    char buf[1024];
    regex_t r;
    int options = REG_EXTENDED, i;
    bool go;
    string tmp;

    showprogress(fname);

    if(keypressed()) {
        getkey();
        return false;
    }
    
    if(!casesens) options |= REG_ICASE;

    switch(mmode) {
        case maregex: go = !regcomp(&r, mask.c_str(), options); break;
        case masub: go = true; break;
    }

    if(go) {
        if(f = fopen(fname.c_str(), "r")) {
            for(l = 1; !feof(f); l++) {
                freads(f, buf, 1024);

                switch(mmode) {
                    case maregex: go = !regexec(&r, buf, 0, 0, 0); break;
                    case masub:
                        tmp = mask;
                        if(!casesens) {
                            for(i = 0; i < strlen(buf); i++) buf[i] = toupper(buf[i]);
                            for(i = 0; i < tmp.size(); i++) tmp[i] = toupper(tmp[i]);
                        }
                        go = strstr(buf, tmp.c_str());
                        break;
                }

                if(go) {
                    grepresult gr;
                    gr.fname = fname;
                    gr.line = l;
                    results.push_back(gr);
                }
            }
            fclose(f);
            gentree();
            tree->redraw();
        }

        if(mmode == maregex) regfree(&r);
    }

    return true;
}

bool motorgrepper::grepconnected(motorproject *pr) {
    int i, k;
    filefolder *fold;
    linkedlist *flist;
    projectfile *pf;
    bool fin = false;

    for(i = 0; i < pr->tmpl->getfoldercount() && !fin; i++) {
        fold = pr->tmpl->getfolder(i);
        if(fold->getelem() == fefile) {
            flist = pr->getfiles(fold);
            for(k = 0; k < flist->count && !fin; k++) {
                pf = (projectfile *) flist->at(k);
                fin = !grepfile(pr->absfname(pf->getfname()));
            }
        }
    }

    return !fin;
}

bool motorgrepper::grepfolder(filefolder *fold) {
    int j;
    bool fin = false;
    projectfile *pf;
    linkedlist *flist = proj->getfiles(fold);

    for(j = 0; j < flist->count && !fin; j++) {
        pf = (projectfile *) flist->at(j);
        fin = !grepfile(pf->getfname());
    }

    return !fin;
}

void motorgrepper::perform() {
    int i, j;
    filefolder *fold;
    projectfile *pf;
    linkedlist *flist;
    bool fin = false;
    
    if(mask.size()) {
        initial = false;

        results.clear();
        tree->clear();
        tree->menu.setpos(listpos = 0);

        switch(mode) {
            case gmcurrent:
                grepfile(ed.getcurrentfname());
                break;
            case gmall:
            case gmfolders:
                for(i = 0; i < proj->tmpl->getfoldercount() && !fin; i++) {
                    fold = proj->tmpl->getfolder(i);

                    if(fold->getelem() == fefile)
                    if(mode == gmall)
                    if(find(exclfold.begin(), exclfold.end(), i+1) == exclfold.end())
                        fin = !grepfolder(fold);
                }
                break;
            case gmconnected:
                for(i = 0; i < proj->tmpl->getfoldercount() && !fin; i++) {
                    fold = proj->tmpl->getfolder(i);

                    if(fold->getelem() == feproj)
                    if(flist = proj->getfiles(fold))
                    for(j = 0; j < flist->count && !fin; j++) {
                        pf = (projectfile *) flist->at(j);
                        if(find(exclfold.begin(), exclfold.end(), i+1) == exclfold.end())
                            fin = !grepconnected(plist.get(pf->getfname()));
                    }
                }
                break;
        }
        initbar();
    }
}

void motorgrepper::gentree() {
    int i, k;
    bool found;
    filefolder *fold;
    projectfile *pf;
    linkedlist *flist;

    tree->clear();

    if(initial) {
        int ns = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Search "));
        
        tree->addleaff(ns, 0, 0, _(" Look for : %s "), mask.c_str());
        tree->addleaff(ns, 0, 0, _(" Mode : %s "), strmaskmode(mmode));
        tree->addleaff(ns, 0, 0, _(" Case sensitive : %s "), stryesno(casesens));
        tree->addleaff(ns, 0, 0, _(" Look in : %s "), strgrepmode(mode));

        if(mode == gmfolders) {
            int nf = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Folders "));

            for(i = 0; i < proj->tmpl->getfoldercount(); i++) {
                fold = proj->tmpl->getfolder(i);

                if(fold->getelem() == fefile)
                    tree->addleaff(nf, conf.getcolor(cp_menu), (void *) (i+1),
                        " [%c] %s ",
                        find(exclfold.begin(), exclfold.end(), i+1) !=
                            exclfold.end() ? ' ' : 'X',
                        fold->getname().c_str());
            }
        } else if(mode == gmconnected) {
            int nc = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Projects "));

            for(i = 0; i < proj->tmpl->getfoldercount(); i++) {
                fold = proj->tmpl->getfolder(i);

                if(fold->getelem() == feproj) {
                    flist = proj->getfiles(fold);
                    for(k = 0; k < flist->count; k++) {
                        pf = (projectfile *) flist->at(k);
                
                        tree->addleaff(nc, conf.getcolor(cp_menu), (void *) (k+1), " [%c] %s ",
                        find(exclfold.begin(), exclfold.end(), i+1) !=
                        exclfold.end() ? ' ' : 'X', pf->getfname().c_str());
                    }
                }
            }
        }
    } else {
        int nm = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Matches found "));

        for(vector<grepresult>::iterator ri = results.begin(); ri != results.end(); ri++)
            tree->addleaf(nm, 0, 0, " " + ri->fname + ":" + i2str(ri->line) + " ");
    }

    if(genfirst) {
        tree->menu.setpos(listpos);
        genfirst = false;
    }
}

void motorgrepper::exec() {
    bool finished = false;
    int n, i, foldnum;
    string tmp;

    w = new textwindow(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT,
        conf.getcolor(cp_menufr), TW_CENTERED,
        conf.getcolor(cp_menuhl), _(" Motor grepper "));

    w->open();

    tree = new treeview(w->x1+1, w->y1+1, w->x2, w->y2-2,
        conf.getcolor(cp_menu), conf.getcolor(cp_menusel),
        conf.getcolor(cp_menu), conf.getcolor(cp_menu));

    initbar();
    tree->menu.otherkeys = &listkeys;
    genfirst = true;

    if(initial) {
        mask = "";
        tmp = ed.atcursor();
    }

    while(!finished) {
        gentree();

        if(mask.size()) {
            foldnum = (int) tree->open(&n);
        } else {
            mask = tmp;
            
            tree->menu.setpos(1);
            tree->redraw();
            n = 2;
        }
        
        if(n) {
            switch(bar->item) {
                case 0:
                    if(initial)
                    switch(n) {
                        case 2:
                            tmp = core.input(mask, _("Look for: "));
                            if(!tmp.size()) {
                                finished = true;
                            } else {
                                mask = tmp;
                            }
                            break;
                        case 3:
                            mmode =
                                mmode == masub ? maregex : masub;
                            break;
                        case 4: casesens = !casesens; break;
                        case 5:
                            exclfold.clear();
                            mode =
                                mode == gmcurrent ? gmall :
                                mode == gmall ? gmfolders :
                                mode == gmfolders ? gmconnected :
                                gmcurrent;
                            break;
                        default:
                            if(n > 6) {
                                vector<int>::iterator ei = find(exclfold.begin(), exclfold.end(), foldnum);
                                if(ei != exclfold.end()) exclfold.erase(ei);
                                else exclfold.push_back(foldnum);
                            }
                            break;
                    } else {
                       if(n-2 < results.size()) {
                           ed.openfile(results[n-2].fname, results[n-2].line);
                           listpos = tree->menu.getpos();
                           finished = true;
                       }
                    }
                    break;
                case 1:
                    if(initial) {
                        perform();
                    } else {
                        initial = true;
                        initbar();
                    }
                    break;
            }
        } else {
            finished = true;
        }
    }

    if(initial) {
        listpos = tree->menu.getpos();
    }

    w->close();
    ed.redraw();

    delete w;
}

void motorgrepper::nextresult() {
    if(!initial)
    if(listpos < results.size()) {
        ed.openfile(results[listpos].fname, results[listpos].line);
        ed.redraw();
        listpos++;
    }
}

int motorgrepper::listkeys(verticalmenu *m, int k) {
    switch(k) {
        case KEY_RIGHT:
        case KEY_LEFT: grep.bar->movebar(k); break;
        case '\r': return m->getpos()+1;
    }
    return -1;
}

