#include "manager.h"
#include "conf.h"
#include "ui.h"
#include "tlist.h"
#include "plist.h"
#include "edit.h"
#include "tagbrowser.h"

#define NO_CVSCHECK 2

CMotorManager::CMotorManager() {
    ctree = 0;
}

CMotorManager::~CMotorManager() {
}

// --------------------------------------------------------------------------

void CMotorManager::gensettings(treeview *tree) {
    char *p;

    tree->clear();

    int nmain = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Main "));
    int ncomp = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Compilation "));
    int ncvs  = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" CVS "));
    int nmake  = tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Make "));

    tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Files.. "));
    tree->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Directories.. "));

    tree->addleaff(nmain, 0, 0, _(" Result file name : %s "), proj->result.c_str());
    tree->addleaff(nmain, 0, 0, _(" Template : %s "), proj->tmpl->getname().c_str());
    tree->addleaff(nmain, 0, 0, _(" Root directory : %s "), proj->rootdir.c_str());
    tree->addleaff(nmain, 0, 0, _(" Out directory : %s "), proj->outdir.c_str());

    tree->addleaff(ncomp, 0, 0, _(" Compiler options : %s "), proj->compoptions.c_str());
    tree->addleaff(ncomp, 0, 0, _(" Linker options : %s "), proj->linkoptions.c_str());

    tree->addleaff(ncvs, 0, 0,  _(" Module : %s "), proj->cvsmodule.c_str());
    tree->addleaff(ncvs, 0, 0,  _(" Repository : %s "), proj->cvsroot.c_str());

    tree->addleaff(nmake, 0, 0, _(" Leave Makefile.motor : %s "), stryesno(proj->leavemakef));
    tree->addleaff(nmake, 0, 0, _(" Make command replacement : %s "), proj->altermake.c_str());
    tree->addleaff(nmake, 0, 0, _(" Execute after make : %s "), proj->aftermake.c_str());
}

void CMotorManager::changesetting(string *param, string prompt, bool emptyallowed) {
    string ans = core.input(*param, prompt);
    if(!ans.empty() || emptyallowed) *param = ans;
}

void CMotorManager::changetemplate() {
    motortemplate *newt;
    
    if((newt = tlist.select(proj->tmpl)) != proj->tmpl)
    if(core.ask("YN", _("Changing template needs rearranging files in folders. Proceed?")) == ANSWER_YES) {
        proj->migratefiles(newt);
    }
}

bool CMotorManager::settings() {
    int n, i, b;
    string ans;
    dialogbox db;
    
    textwindow w(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT,
        conf.getcolor(cp_menufr), TW_CENTERED,
        conf.getcolor(cp_menuhl), _(" Project settings "));

    db.setwindow(&w, false);
    db.settree(new treeview(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel), conf.getcolor(cp_menu),
        conf.getcolor(cp_menu)));
    db.setbar(new horizontalbar(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel), _("Change"), 0));

    core.fm->setstartpoint(proj->rootdir.c_str());
    bool finished = false;

    while(!finished) {
        gensettings(db.gettree());
        finished = !db.open(n, b);
        
        if(!finished)
        switch(n) {
            case  2: changesetting(&proj->result, _("result file name: "), false); break;
            case  3: changetemplate(); break;
            case  4:
                core.fm->chroot = false;
                ans = core.inputdir(proj->rootdir, _("root directory: "));
                if(!ans.empty()) {
                    if(!access(ans.c_str(), F_OK)) proj->rootdir = ans;
                    else core.bottom(_("Root dir must exist"));
                }
                break;

            case  5:
                core.fm->chroot = false;
                proj->outdir = core.inputdir(proj->outdir, _("out directory: "));
                if(proj->outdir.empty()) proj->outdir = proj->rootdir;
                break;

            case  7: changesetting(&proj->compoptions, _("additional compiler command line options: "), true); break;
            case  8: changesetting(&proj->linkoptions, _("additional linker command line options: "), true); break;
            case 10: changesetting(&proj->cvsmodule, _("cvs module name: "), true); break;

            case 11:
                if(i = core.cvsrootselect(true))
                    cvs.setrepos(proj->cvsroot = i == 1 ? "" : conf.getrepos(i-2));
                break;

            case 13: proj->leavemakef = !proj->leavemakef; break;
            case 14: changesetting(&proj->altermake, _("make command replacement: "), true); break;
            case 15: changesetting(&proj->aftermake, _("execute after make: "), true); break;
            case 16: finished = files(); break;
            case 17: finished = dirs(); break;
        }
    }

    db.close();
    return n;
}

// --------------------------------------------------------------------------

void CMotorManager::performaddfile(string fname, filefolder *dest, motorproject *source, bool add) {
    string p;
    
    switch(dest->getelem()) {
        case fefile:
        case fedir:
            p = source->absfname(fname);
            break;
        default:
            p = fname;
            break;
    }

    if(add) proj->addfile(p, dest); else
    proj->removefile(p, dest->getelem());
}

void CMotorManager::performtakes(motorproject *mp, filefolder *fold, bool add) {
    int i, k, j;
    char *pout;
    string take, ptemplate, destfolder, buf, sub, token;
    filefolder *dest, *source;
    projectfile *pf;
    linkedlist flist;

    pparamslist *params = pparamslist_create();
    mp->populateparselist(params);

    for(i = 0; i < fold->gettakecount(); i++) {
        take = fold->gettake(i);
        ptemplate = destfolder = "";

        buf = pout = sstrparse(take.c_str(), params);
        delete pout;

        for(sub = buf; sub.size(); ) {
            token = getwordquote(sub);
            if(token == "from") ptemplate = getwordquote(sub); else
            if(token == "to") destfolder = getwordquote(sub);
        }

        if(!ptemplate.empty() && !destfolder.empty())
        if(dest = proj->tmpl->getfolder(destfolder)) {
            sub = getwordquote(buf);

            if(sub.substr(0, 1) == "\"") {
                performaddfile(getword(sub, "\""), dest, mp, add);
            } else if(source = mp->tmpl->getfolder(sub)) {
                for(j = 0; j < mp->getfilecount(); j++) {
                    pf = mp->getfile(j);
                    if(pf->getfolder() == source)
                        performaddfile(pf->getfname(), dest, mp, add);
                }
            }
        }
    }

    pparamslist_free(params);
}

bool CMotorManager::addfile(filefolder *fold) {
    bool ret = false;
    string fname;
    folderelement elem;
    motorproject *mp;

    core.fm->chroot = true;

    switch(elem = fold->getelem()) {
        case fefile: fname = core.inputfile("", _("file to add: ")); break;
        case fedir:  fname = core.inputdir("", _("directory to add: ")); break;
        case fesymb: fname = core.input("", _("symbol to add: ")); break;
        case feproj:
            if(mp = plist.select(fold)) {
                performtakes(mp, fold, true);
                fname = mp->name;
            }
            break;
    }

    core.fm->chroot = false;

    if(!fname.empty()) {
        if(elem == fefile)
        if(proj->cvsenabled()) {
            int askr = core.askf("YNC", _("add %s under the CVS control?"), justfname(fname.c_str()));
            bool finished = !(askr == ANSWER_YES);
            proj->chdir();

            while(!finished) {
                if(access(fname.c_str(), F_OK)) {
                    FILE *f = fopen(fname.c_str(), "w");
                    fclose(f);
                }

                cvs.add(proj->rootdir, proj->relativefname(fname));

                if(!(finished = cvs.result == rkok)) {
                    finished = (core.ask("YN", _("cvs add command wasn't successfull. retry?")) == ANSWER_NO);
                }
            }
        }

        proj->addfile(fname, fold);
        ret = true;
    }

    return ret;
}

void CMotorManager::removefile(projectfile *pf) {
    folderelement elem;
    motorproject *mp;

    switch(elem = pf->getfolder()->getelem()) {
        case fefile:
            cvs.remove(proj->rootdir, pf->getfname(), false);
        case feproj:
            if(mp = plist.get(pf->getfname()))
                performtakes(mp, pf->getfolder(), false);
            break;
    }

    proj->removefile(pf->getfname(), elem);
}

bool CMotorManager::viewconnected(motorproject *mp) {
    int i, k, bc = 0;
    projectfile *pf;
    bool ret = false;

    textwindow w(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT, conf.getcolor(cp_menufr), TW_CENTERED);
    w.set_title(conf.getcolor(cp_menuhl), _(" %s connected project files "), mp->name.c_str());
    w.open();

    verticalmenu *m = new verticalmenu(w.x1+1, w.y1+1, w.x2, w.y2,
    conf.getcolor(cp_menu), conf.getcolor(cp_menusel));

    for(i = 0; i < mp->getfilecount(); i++) {
        pf = mp->getfile(i);
        if(pf->getfolder()->getelem() == fefile)
            m->additem(" " + pf->getfname());
    }

    if(k = m->open())
    for(i = bc = 0; i < mp->getfilecount(); i++) {
        pf = mp->getfile(i);
        if(pf->getfolder()->getelem() == fefile)
        if(++bc == k) {
            ed.openfile(mp->absfname(pf->getfname()), 0, true);
            ret = true;
            break;
        }
    }
    
    w.close();
    delete m;
    return ret;
}

void CMotorManager::gensourcetree(treeview *tree, bool dirs) {
    bool first = tree->getcount() == 1;
    int i, k, nfolder, nitem;
    filefolder *f;
    projectfile *pf;

    linkedlist *flist;
    vector<bool> opennodes;

    for(i = 0; i < tree->menu.getcount(); i++) {
        k = tree->getid(i);
        if(tree->isnode(k)) {
            opennodes.push_back(tree->isnodeopen(k));
        }
    }

    tree->clear();
    nitem = 0;

    for(i = 0; i < proj->tmpl->getfoldercount(); i++) {
        f = proj->tmpl->getfolder(i);

        if((dirs && (f->getelem() == fedir)) || (!dirs && (f->getelem() != fedir))) {
            flist = proj->getfiles(f);
            nfolder = tree->addnode(0, conf.getcolor(cp_menuhl), f, " " + f->getname() + " ");

            if((opennodes.size() <= nitem) || opennodes[nitem]) tree->opennode(nfolder);
            nitem++;

            for(k = 0; k < flist->count; k++) {
                pf = (projectfile *) flist->at(k);
                tree->addleaf(nfolder, 0, pf, " " + pf->getfname() + " ");

                if(first)
                if(proj->absfname(pf->getfname()) == proj->absfname(ed.getcurrentfname())) {
                    tree->menu.setpos(tree->getcount()-2);
                }
            }
        }
    }
}

bool CMotorManager::projectfiles(bool dirs) {
    int i, n, curid, foldid, b;
    filefolder *fold;
    motorproject *mp;
    bool ret = false;
    dialogbox db;
    
    textwindow w(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT, conf.getcolor(cp_menufr), TW_CENTERED);
    w.set_title(conf.getcolor(cp_menuhl), dirs ? _(" Project directories ") : _(" Project files "));

    db.setwindow(&w, false);
    db.settree(new treeview(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel), conf.getcolor(cp_menu),
        conf.getcolor(cp_menu)));
    db.gettree()->collapsable = true;

    if(dirs) {
        db.setbar(new horizontalbar(conf.getcolor(cp_menu),
            conf.getcolor(cp_menusel), _("Add"), _("Remove"), 0));
    } else {
        db.setbar(new horizontalbar(conf.getcolor(cp_menu),
            conf.getcolor(cp_menusel), _("Add"), _("Remove"), _("Edit"), 0));
        db.getbar()->item = 2;
    }

    db.addkey(KEY_IC, 0);
    db.addkey(KEY_DC, 1);
    db.gettree()->menu.setpos(-1);

    core.fm->setstartpoint(proj->rootdir.c_str());
    bool finished = false;

    while(!finished) {
        gensourcetree(db.gettree(), dirs);
        finished = !db.open(n, b);

        if(!finished) {
            curid = db.gettree()->getid(db.gettree()->menu.getpos());

            if(db.gettree()->isnode(curid)) foldid = curid;
            else foldid = db.gettree()->getparent(curid);
            
            switch(b) {
                case 0:
                    if(fold = (filefolder *) db.gettree()->getref(foldid))
                    if(addfile(fold)) db.gettree()->opennode(foldid);
                    break;
                case 1:
                    if(curid != foldid)
                    removefile((projectfile *) db.gettree()->getref(curid));
                    break;
                case 2:
                    if(curid == foldid) {
                        if(!db.gettree()->isnodeopen(curid)) db.gettree()->opennode(curid);
                        else db.gettree()->closenode(curid);
                    } else {
                        proj->chdir();
                        projectfile *pf = (projectfile *) db.gettree()->getref(curid);

                        switch(pf->getfolder()->getelem()) {
                            case fefile:
                                ed.openfile(proj->absfname(pf->getfname()), 0, true);
                                finished = true;
                                break;
                            case feproj:
                                if(mp = plist.get(pf->getfname()))
                                finished = viewconnected(mp);
                                break;
                        }
                        
                        ret = true;
                    }
                    break;
            }
        } else {
            ret = false;
        }
    }
    
    db.close();
    core.fm->chroot = false;

    return ret;
}

bool CMotorManager::files() {
    return projectfiles(false);
}

bool CMotorManager::dirs() {
    return projectfiles(true);
}

void CMotorManager::generatefiles(bool all = true) {
    bool fin;
    int i, b;
    dialogbox db;
    linkedlist ngen;
    string fname;
    pparamslist *params = pparamslist_create();
    char *pfn;
    
    ngen.freeitem = &nothingfree;
    proj->populateparselist(params);

    db.setwindow(new textwindow(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT,
        conf.getcolor(cp_menufr), TW_CENTERED, conf.getcolor(cp_menuhl),
        _(" Generate files from the template %s "),
        proj->tmpl->getname().c_str()));

    db.setmenu(new verticalmenu(conf.getcolor(cp_menu), conf.getcolor(cp_menusel)));

    db.setbar(new horizontalbar(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel), _("Mark"), _("Generate"), 0));

    db.addkey(' ', 0);

    for(fin = false; !fin;) {
        db.getmenu()->clear();
        
        for(i = 0; i < proj->tmpl->getftemplcount(); i++) {
            fname = proj->tmpl->getftempl(i);

            pfn = sstrparse(fname.c_str(), params);
            pfn[strlen(pfn)-5] = 0;

            db.getmenu()->additemf(" [%c] %s", in_numberlist(&ngen, i+1) ? 'X' : ' ', pfn);
        }

        if(!(fin = !db.open(i, b)))
        switch(b) {
            case 0:
                if(in_numberlist(&ngen, i)) del_numberlist(&ngen, i);
                else ngen.add((void *) i);
                break;
            case 1:
                ed.editclose(CA_DONTASK | CA_DONTSHOW);
                proj->generatesource(&ngen);
                ed.editload();
                fin = true;
                break;
        }
    }

    db.close();
    pparamslist_free(params);
    ed.redraw();
}

// --------------------------------------------------------------------------

bool CMotorManager::closeprev(int options = 0) {
    bool ret = true;
    
    if(proj) {
        if(!proj->getedcount())
        if(ret = ed.editclose(CA_DONTSHOW)) proj->save();
        
        if(options & NO_CVSCHECK) {
        } else {
            core.cvscheck(ckcommit);
        }
    }

    return ret;
}

bool CMotorManager::checkopen(motorproject *pr) {
    struct stat st;
    bool ret = false;

    if(stat(pr->rootdir.c_str(), &st) || !S_ISDIR(st.st_mode)) {
        core.bottom(_("Cannot access the project root directory"));
    } else {
        core.bottomf(_("Project %s has been opened"), pr->name.c_str());
        ret = true;
    }

    return ret;
}

bool CMotorManager::performcreate(screateinfo *info) {
    bool ret = true;

    if(info->rootdir.empty()) ret = false, core.bottom(_("Project root directory must be set"));
    if(info->pname.empty()) ret = false, core.bottom(_("Project name must be set"));
    if(!info->t) ret = false, core.bottom(_("Template must be set"));

    if(ret) {
        switch(info->mode) {
            case cmfiles: break;
            default:
                if(access(info->rootdir.c_str(), F_OK))
                if(core.ask("YNC", _("root directory you specified does not exist. create?")) == ANSWER_YES)
                mkdir(info->rootdir.c_str(), S_IRWXU); else ret = false;

                if(ret && access(info->rootdir.c_str(), R_OK | W_OK | X_OK))
                ret = false, core.bottom(_("Access denied for the directory specified"));
                break;
        }

        if(ret) {
            switch(info->mode) {
                case cmfiles: if(access(info->rootdir.c_str(), F_OK)) ret = false, core.bottom(_("Project root directory must exist to use this creation mode")); break;
                case cmcvs: if(info->cvsroot.empty() || info->cvsmodule.empty()) ret = false, core.bottom(_("CVS repository and module name must be specified to use this creation mode")); break;
            }

            if(ret)
            if(closeprev()) {
                plist.create(info->pname.c_str(), info->rootdir.c_str(), info->t);
                plist.setcur(info->pname.c_str());
                proj->chdir();

                switch(info->mode) {
                    case cmscratch: break;
                    case cmcvs:
                        cvs.setrepos(proj->cvsroot = info->cvsroot);
                        cvs.checkout(proj->cvsmodule = info->cvsmodule, proj->rootdir);
                    default:
                        proj->gatherfiles();
                        break;
                }
                
                if(info->gnudoc) proj->addgnudoc();
                if(info->gensource && (info->mode == cmscratch)) proj->generatesource();
                proj->save();
                ret = true;
            }
        }
    }

    return ret;
}

bool CMotorManager::create() {
    int n, b;
    bool ret = true, finished = false;
    screateinfo createinfo;
    dialogbox db;

    db.setwindow(new textwindow(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT,
        conf.getcolor(cp_menufr), TW_CENTERED, conf.getcolor(cp_menuhl),
        _(" Create a new project ")));

    db.setbar(new horizontalbar(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel), _("Settings"), _("Change"), _("Create"), 0));

    db.settree(new treeview(conf.getcolor(cp_menu), conf.getcolor(cp_menusel),
        conf.getcolor(cp_menu), conf.getcolor(cp_menu)));

    db.getbar()->item = 1;

    while(!finished) {
        db.gettree()->clear();
        int nopt, nproj = db.gettree()->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Project "));

        db.gettree()->addleaff(nproj, 0, 0, _(" Creation mode : %s "),
            createinfo.mode == cmscratch ? _("from scratch") :
            createinfo.mode == cmfiles ? _("from files") :
            _("from cvs"));
        
        db.gettree()->addleaff(nproj, 0, 0, _(" Template : %s "), createinfo.t ? createinfo.t->getname().c_str() : "none");
        db.gettree()->addleaff(nproj, 0, 0, _(" New project name : %s "), createinfo.pname.c_str());
        db.gettree()->addleaff(nproj, 0, 0, _(" Root directory : %s "), createinfo.rootdir.c_str());
        db.gettree()->addleaff(nproj, 0, 0, _(" GNU standard documentation : %s "), stryesno(createinfo.gnudoc));

        if(createinfo.mode != cmfiles) {
            nopt = db.gettree()->addnode(0, conf.getcolor(cp_menuhl), 0, _(" Creation "));

            switch(createinfo.mode) {
                case cmscratch:
                    db.gettree()->addleaff(nopt, 0, 0, _(" Generate inital source: %s "), stryesno(createinfo.gensource));
                    break;
                case cmcvs:
                    db.gettree()->addleaff(nopt, 0, 0, _(" CVS repository : %s "), createinfo.cvsroot.c_str());
                    db.gettree()->addleaff(nopt, 0, 0, _(" Module name : %s "), createinfo.cvsmodule.c_str());
                    break;
            }
        }

        if(!db.open(n, b)) {
            ret = false;
            finished = true;
            continue;
        }

        switch(b) {
            case 0: conf.execute(); break;
            case 1:
                switch(n) {
                    case 2:
                        createinfo.mode =
                        createinfo.mode == cmscratch ? cmfiles :
                        createinfo.mode == cmfiles ? cmcvs :
                        cmscratch;
                        break;
                    case 3: createinfo.t = tlist.select(createinfo.t); break;
                    case 4:
                        createinfo.pname = core.input(createinfo.pname, _("project name: "));
                        
                        if(!createinfo.pname.empty()) {
                            if(createinfo.rootdir.empty()) createinfo.rootdir = (string) getenv("HOME") + "/" + createinfo.pname;
                            if(createinfo.cvsmodule.empty()) createinfo.cvsmodule = createinfo.pname;
                        }
                        break;
                        
                    case 5:
                        createinfo.rootdir = core.inputdir(createinfo.rootdir, _("root directory: "));
                        if(!createinfo.rootdir.empty() && createinfo.pname.empty()) {
                            createinfo.pname = justfname(createinfo.rootdir.c_str());
                        }
                        break;
                        
                    case 6: createinfo.gnudoc = !createinfo.gnudoc; break;

                    default:
                        if(createinfo.mode == cmscratch)
                        switch(n) {

                            case 8: createinfo.gensource = !createinfo.gensource;
                            
                        } else if(createinfo.mode == cmcvs)
                        switch(n) {
                        
                            case 8:
                                if(n = core.cvsrootselect(false))
                                    createinfo.cvsroot = conf.getrepos(n-1);
                                break;

                            case 9:
                                createinfo.cvsmodule = core.input(createinfo.cvsmodule, _("cvs module name: "));
                                break;
                                
                        }
                }
                break;

            case 2: finished = performcreate(&createinfo); break;
        }
    }

    db.close();
    return ret;
}

void CMotorManager::list() {
    int i, j, tnode, b;
    bool curremoved = false;
    motorproject *pr, *lastpr = 0;
    motortemplate *tp;
    string dname, buf;
    dialogbox db;

    dname = (string) getenv("HOME") + "/.motor/lastproj";

    ifstream f(dname.c_str());
    if(f.is_open()) {
        getline(f, buf);
        f.close();
        lastpr = plist.get(buf);
    }

    db.setwindow(new textwindow(0, 0, (int) (DIALOG_WIDTH*0.8),
        DIALOG_HEIGHT, conf.getcolor(cp_menufr), TW_CENTERED,
        conf.getcolor(cp_menuhl), _(" Projects registry ")));
    db.settree(new treeview(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel), conf.getcolor(cp_menu),
        conf.getcolor(cp_menu)));
    db.setbar(new horizontalbar(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel),
        _("Settings"), _("Add"), _("Remove"), _("Open"), 0));

    db.addkey(KEY_IC, 1);
    db.addkey(KEY_DC, 2);

    db.getbar()->item = 3;
    if(proj) proj->edclear();

    bool finished = false, first = true;

    while(!finished) {
        db.gettree()->clear();
    
        for(i = 0; i < tlist.count; i++) {
            tp = tlist.get(i);
            tnode = -1;

            for(j = 0; j < plist.count; j++) {
                pr = plist.get(j);
            
                if(pr->tmpl == tp) {
                    if(tnode == -1) {
                        tnode = db.gettree()->addnode(0, conf.getcolor(cp_menuhl), 0, " " + tp->getname() + " ");
                        db.gettree()->opennode(tnode);
                    }
                    
                    db.gettree()->addleaf(tnode, 0, pr, " " + pr->name + " ");

                    if(first)
                    if((proj && (pr == proj)) || (!proj && (pr == lastpr))) {
                        db.gettree()->menu.setpos(db.gettree()->getcount()-2);
			first = false;
                    }
                    
                }
            }
        }

        if(plist.count) {
            if(finished = !db.open(i, b, (void **) &pr)) break;
        } else {
            b = 1;
            // create new project
        }

        switch(b) {
            case 0: conf.execute(); break;
            case 1:
                finished = create();
                if(!plist.count && !finished) finished = true;
                break;
            case 2:
                if(pr)
                if(closeprev(NO_CVSCHECK))
                switch(core.ask("YNC", _("delete all the project files as well?"))) {
                    case ANSWER_YES:
                        chdir(getenv("HOME"));
                        if(!fork()) {
                            execlp("/bin/rm", "/bin/rm", "-rf", pr->rootdir.c_str(), 0);
                            _exit(0);
                        }
                    case ANSWER_NO:
                        dname = (string) getenv("HOME") + "/.motor/projects/" + pr->name;
                        unlink(dname.c_str());
                        browser.cleanup(pr->name);

                        if(proj)
                        if(pr->name == proj->name) {
                            curremoved = curremoved || true;
                            ed.editclose(CA_DONTASK | CA_DONTSHOW);
                            proj = 0;
                        }

                        for(i = 0; i < plist.count; i++)
                        if(plist.get(i) == pr) plist.remove(i);
                        
                        break;
                    case ANSWER_CANCEL:
                        break;
                }
                break;
                    
            case 3:
                if(pr)
                if(checkopen(pr))
                if(closeprev()) {
                    if(plist.setcur(pr->name)) {
                        core.cvscheck(ckupdate);
                        finished = true;
                    }
                }
                break;
        }
    }

    db.close();

    if(proj) {
        ed.editload();
        ed.redraw();
        ed.status();
        if(proj->watchactive) core.openwatch(false);
    }
}

