#include "project.h"
#include "conf.h"
#include "ui.h"
#include "edit.h"
#include "tlist.h"
#include "plist.h"
#include "debug.h"

installrule::installrule(string nfname, string ndest) {
    fname = nfname;
    dest = ndest;
}

installrule::~installrule() {
}

string installrule::getfname() {
    return fname;
}

string installrule::getdest() {
    return dest;
}

// --------------------------------------------------------------------------

projectfile::projectfile(filefolder *f, string name = ""):
fname(name), folder(f) {
}

projectfile::~projectfile() {
}

string projectfile::getfname() const {
    return fname;
}

filefolder *projectfile::getfolder() const {
    return folder;
}

// --------------------------------------------------------------------------

motorproject *grabpr = 0;
makenamemode currentmakemode = mmlocal;

motorproject::motorproject() {
    tmpl = 0;
    pautoconf = pconnected = paddinfo = watchactive = leavemakef = false;

    templist.freeitem = &nothingfree;
    ineditor.freeitem = &charpointerfree;
}

motorproject::~motorproject() {
    delete tmpl;
}

void motorproject::loadfromfile(string fname) {
    string sect, buf, sub;

    if(!fname.empty())
    if(fname[0] != '/') {
        fname = (string) getenv("HOME") + "/.motor/projects/" + fname;
    }

    ifstream f(fname.c_str());
    if(f.is_open()) {
        while(getconf(sect, buf, f)) {
            if(sect == "template") tmpl = tlist.get(buf); else
            if(sect == "binary") result = buf; else
            if(sect == "outdir") outdir = buf; else
            if(sect == "rootdir") {
                rootdir = buf;
                char dbuf[1024];
                chdir();
                getcwd(dbuf, 1024);
                arootdir = dbuf;
            } else
            if(sect == "args") args = buf; else
            if(sect == "cvsrepos") cvsroot = buf; else
            if(sect == "cvsmodule") cvsmodule = buf; else
            if(sect == "altermake") altermake = buf; else
            if(sect == "aftermake") aftermake = buf; else
            if(sect == "packinstcmd") packinstcmd = buf; else
            if(sect.substr(0, 7) == "folder=") {
                addfile(buf, tmpl ? tmpl->getfolder(sect.substr(7)) : 0);
            } else
            if(sect == "desktop") watchactive = (buf == "w"); else
            if(sect == "debug")
                switch(buf[0]) {
                    case 'W': dpoints.push_back(debugpoint(dpwatch, buf.substr(2))); break;
                    case 'B': dpoints.push_back(debugpoint(dpbreakpoint, buf.substr(2))); break;
            } else
            if(sect == "ocompiler") compoptions = buf; else
            if(sect == "olinker") linkoptions = buf; else
            if(sect == "packver") version = buf; else
            if(sect == "packoutdir") packgendir = buf; else
            if(sect == "packautoconf") pautoconf = (bool) atol(buf.c_str()); else
            if(sect == "leavemakef") leavemakef = (bool) atol(buf.c_str()); else
            if(sect == "installrules") {
                sub = getword(buf, "|");
                addinst(sub, buf);
            } else
            if(sect == "packconnected") pconnected = (bool) atol(buf.c_str()); else
            if(sect == "packinfo") paddinfo = (bool) atol(buf.c_str()); else
            if(sect == "editor") edadd(buf.c_str());
        }

        f.close();
    }

    if(version.empty()) version = "0.1";
    if(packgendir.empty()) packgendir = rootdir;
    if(outdir.empty()) outdir = rootdir;
}

void motorproject::save() {
    linkedlist *flist;
    vector<debugpoint>::iterator di;
    vector<string> processed;
    FILE *f;
    int i, k;

    if(this == proj) getdebug();
    string pname = (string) getenv("HOME") + "/.motor/projects/" + name;

    if(f = fopen(pname.c_str(), "w")) {
        fprintf(f, "%%template\n%s\n", tmpl->getname().c_str());
        fprintf(f, "%%binary\n%s\n", result.c_str());
        fprintf(f, "%%outdir\n%s\n", outdir.c_str());
        fprintf(f, "%%rootdir\n%s\n", rootdir.c_str());
        fprintf(f, "%%args\n%s\n", args.c_str());
        fprintf(f, "%%cvsrepos\n%s\n", cvsroot.c_str());
        fprintf(f, "%%cvsmodule\n%s\n", cvsmodule.c_str());
        fprintf(f, "%%ocompiler\n%s\n", compoptions.c_str());
        fprintf(f, "%%olinker\n%s\n", linkoptions.c_str());
        fprintf(f, "%%altermake\n%s\n", altermake.c_str());
        fprintf(f, "%%aftermake\n%s\n", aftermake.c_str());
        fprintf(f, "%%packinstcmd\n%s\n", packinstcmd.c_str());

        fprintf(f, "%%desktop\n");
        if(watchactive) fprintf(f, "w\n");

        for(i = 0; i < tmpl->getfoldercount(); i++) {
            filefolder *fold = tmpl->getfolder(i);
            
            if(find(processed.begin(), processed.end(), fold->getname()) == processed.end()) {
                fprintf(f, "%%folder=%s\n", fold->getshortname().c_str());
                for(k = 0, flist = getfiles(fold); k < flist->count; k++)
                    fprintf(f, "%s\n", ((projectfile *) flist->at(k))->getfname().c_str());

                processed.push_back(fold->getname());
            }
        }

        fprintf(f, "%%packver\n%s\n", version.c_str());
        fprintf(f, "%%packoutdir\n%s\n", packgendir.c_str());
        fprintf(f, "%%packautoconf\n%d\n", pautoconf ? 1 : 0);
        fprintf(f, "%%packconnected\n%d\n", pconnected ? 1 : 0);
        fprintf(f, "%%packinfo\n%d\n", paddinfo ? 1 : 0);
        fprintf(f, "%%leavemakef\n%d\n", leavemakef ? 1 : 0);

        for(i = 0, fprintf(f, "%%editor\n"); i < getedcount(); i++)
        fprintf(f, "%s\n", edget(i).c_str());

        for(i = 0, fprintf(f, "%%installrules\n"); i < getinstcount(); i++)
        fprintf(f, "%s|%s\n", getinst(i)->getfname().c_str(), getinst(i)->getdest().c_str());

        fprintf(f, "%%debug\n");

        for(di = dpoints.begin(); di != dpoints.end(); di++)
        switch(di->getkind()) {
            case dpwatch:
                fprintf(f, "W\t%s\n", di->getwatch().name.c_str());
                break;
            case dpbreakpoint:
                fprintf(f, "B\t%s\t%d\n",
                        di->getbreakpoint().fname.c_str(),
                        di->getbreakpoint().line
                );
                break;
        }
        
        fclose(f);
    }

    pname = (string) getenv("HOME") + "/.motor/lastproj";
    
    if(f = fopen(pname.c_str(), "w")) {
        fprintf(f, "%s", name.c_str());
        fclose(f);
    }
}

int motorproject::gatherfilesftw(const char *file, const struct stat *sb, int flag) {
    string infofile = "motor.project.info";
    filefolder *fold;
    string pinfo, pdest;
    int i;

    if(flag == FTW_F) {
        if((string) (file+grabpr->rootdir.size()+1) == infofile) {
            core.bottom(_("Motor project information file found"));
            
            pinfo = grabpr->rootdir + "/motor.project.info";
            pdest = (string) getenv("HOME") + "/.motor/projects/" + grabpr->name;
            fmove(pinfo.c_str(), pdest.c_str());

            string srootdir = grabpr->rootdir;
            motortemplate *stmpl = grabpr->tmpl;
            
            grabpr->loadfromfile(grabpr->name.c_str());

            grabpr->rootdir = srootdir;
            grabpr->tmpl = stmpl;
            
            return 1;
        } else {
            grabpr->addfile(file);
        }
    }

    return 0;
}

void motorproject::addgnudoc() {
    FILE *f;
    chdir();
    
    if(access("INSTALL", F_OK))
    if(f = fopen("INSTALL", "w")) fclose(f);
    
    if(access("README", F_OK))
    if(f = fopen("README", "w")) fclose(f);
    
    if(access("COPYING", F_OK)) {
        fcopy(
          ((string) SHAREDIR + "/motor/COPYING").c_str(),
          (rootdir + "/COPYING").c_str()
        );
    }

    if(access("AUTHORS", F_OK))
    if(f = fopen("AUTHORS", "w")) {
        setpwent();
        struct passwd *p = getpwuid(getuid());
        
        char *r = strchr(p->pw_gecos, ',');
        if(r) *r = 0;
        
        fprintf(f, "%s <%s@%s>\n", p->pw_gecos, p->pw_name, getenv("HOSTNAME"));
        fclose(f);
        endpwent();
    }

    if(access("TODO", F_OK))
    if(f = fopen("TODO", "w")) {
        fclose(f);
    }

    if(access("ChangeLog", F_OK))
    if(f = fopen("ChangeLog", "w")) {
        fclose(f);
    }

    addfile("INSTALL");
    addfile("README");
    addfile("COPYING");
    addfile("AUTHORS");
    addfile("TODO");
    addfile("ChangeLog");

    if(cvsenabled()) {
        cvs.add(rootdir, "INSTALL");
        cvs.add(rootdir, "README");
        cvs.add(rootdir, "COPYING");
        cvs.add(rootdir, "AUTHORS");
        cvs.add(rootdir, "TODO");
        cvs.add(rootdir, "ChangeLog");
    }
}

void motorproject::gatherfiles() {
    grabpr = this;
    ftw(rootdir.c_str(), &gatherfilesftw, 5);
}

string motorproject::relativefname(string fname) {
    string pdir, ret = fname;

    if((pdir = fname.substr(0, rootdir.size())) == rootdir)
    if(fname.size() > rootdir.size())
    if(fname[rootdir.size()] == '/') {
        ret.replace(0, rootdir.size(), "");
        while(!ret.empty() && (ret[0] == '/')) ret.replace(0, 1, "");
        return ret;
    }

    if(!arootdir.empty())
    if((pdir = fname.substr(0, arootdir.size())) == arootdir)
    if(fname.size() > arootdir.size())
    if(fname[arootdir.size()] == '/') {
        ret.replace(0, arootdir.size(), "");
        while(!ret.empty() && (ret[0] == '/')) ret.replace(0, 1, "");
    }

    return ret;
}

string motorproject::absfname(string fname) {
    string ret = fname;
    char sdir[1024], buf[1024];
    struct stat sb;
    
    getcwd(sdir, 1024);

    if(!::chdir(rootdir.c_str()))
    if(!stat(fname.c_str(), &sb)) {
        if(S_ISREG(sb.st_mode) && (fname[0] != '/')) {
            ret = rootdir + (string) "/" + fname;
        } else if(S_ISDIR(sb.st_mode)) {
            if(!::chdir(fname.c_str())) ret = getcwd(buf, 2048);
        }
    }

    ::chdir(sdir);
    return ret;
}

string motorproject::getmakename() {
    return rootdir + "/" + MAKEFILE_NAME;
}

void motorproject::chdir() {
    ::chdir(rootdir.c_str());
}

bool motorproject::contains(string fname) {
    projectfile pf(0, relativefname(fname));
    return files.find(&pf, &projectfilecompare);
}

void motorproject::addfile(string fname, filefolder *fold = 0) {
    int i;
    filefolder *f;
    
    if(fold) {
        projectfile *pf = new projectfile(fold, relativefname(fname));
        if(!files.find(pf, &projectfilecompare)) files.add(pf);
        else delete pf;
    } else {
        if(tmpl) for(i = 0; i < tmpl->getfoldercount(); i++) {
            f = tmpl->getfolder(i);
            if(f->accepts(fname.c_str())) {
                addfile(fname, f);
                break;
            }
        } else {
            if(!contains(fname)) {
                files.add(new projectfile(0, relativefname(fname).c_str()));
            }
        }
    }
}

void motorproject::removefile(string fname, folderelement elem) {
    int i;
    projectfile *pf;
    folderelement el = fefile;
    
    for(i = 0; i < getfilecount(); i++) {
        pf = getfile(i);
        if(pf->getfolder()) el = pf->getfolder()->getelem();
        if((pf->getfname() == fname) && (el == elem)) {
            files.remove(i);
            break;
        }
    }
}

void motorproject::removefile(projectfile *pf) {
    removefile(pf->getfname(), pf->getfolder() ? pf->getfolder()->getelem() : fefile);
}

void motorproject::addinst(string nfname, string ndest) {
    insts.add(new installrule(nfname, ndest));
}

void motorproject::removeinst(int n) {
    insts.remove(n);
}

bool motorproject::isconnectedpathname(string pname) {
    int i, k;
    linkedlist *flist;
    motorproject *connectedp;
    filefolder *fold;
    projectfile *pf;
    string root;

    for(i = 0; i < tmpl->getfoldercount(); i++) {
        fold = tmpl->getfolder(i);

        if(fold->getelem() == feproj)
        for(k = 0, flist = getfiles(fold); k < flist->count; k++) {
            pf = (projectfile *) flist->at(k);

            if(connectedp = plist.get(pf->getfname())) {
                root = connectedp->rootdir;
                while(root[root.size()-1] == '/') root.resize(root.size()-1);
                if(!pname.compare(root, 0, root.size())) return true;
            }
        }
    }

    return false;
}

void motorproject::clearmakefiles() {
    int i, k;
    motorproject *connectedp;
    linkedlist *flist;
    projectfile *pf;
    filefolder *fold;

    if(!proj->leavemakef) {
        unlink(getmakename().c_str());
    }

    for(i = 0; i < tmpl->getfoldercount(); i++) {
        fold = tmpl->getfolder(i);

        if(fold->getelem() == feproj)
        for(k = 0, flist = getfiles(fold); k < flist->count; k++) {
            pf = (projectfile *) flist->at(k);

            if(connectedp = plist.get(pf->getfname()))
            if(!connectedp->leavemakef) {
                unlink(connectedp->getmakename().c_str());
            }
        }
    }
}

linkedlist *motorproject::getfiles(filefolder *fold) {
    int i;
    projectfile *pf;
    templist.empty();

    for(i = 0; i < getfilecount(); i++) {
        pf = getfile(i);
        if(pf->getfolder() == fold) templist.add(pf);
    }

    return &templist;
}

projectfile *motorproject::getfile(int i) {
    return (projectfile *) files.at(i);
}

installrule *motorproject::getinst(int n) {
    return (installrule *) insts.at(n);
}

int motorproject::getfilecount() {
    return files.count;
}

int motorproject::getinstcount() {
    return insts.count;
}

int motorproject::getedcount() {
    return ineditor.count;
}

void motorproject::edclear() {
    ineditor.empty();
}

string motorproject::edget(int n) {
    return (const char *) ineditor.at(n);
}

void motorproject::edadd(string fname) {
    ineditor.add(strdup(fname.c_str()));
}

int motorproject::projectfilecompare(void *key, void *current) {
    projectfile *pf1 = (projectfile *) key;
    projectfile *pf2 = (projectfile *) current;

    bool fnamematch = pf1->getfname() == pf2->getfname();
    bool foldermatch = pf1->getfolder() == pf2->getfolder();

    if(!pf1->getfolder() || !pf1->getfolder()) foldermatch = true;

    return !fnamematch || !foldermatch;
}

char *motorproject::svmakefile(const char *value, const char *key) {
    char *r, *ret = 0;
    buildrule *br;
    string ts;
    
    if(key)
    if(key[0]) {
        if(!strcmp(key, "toobject")) {

            if(r = strrchr(value, '.')) {
                ret = strdup(value);
                ret[r-value] = 0;
                strcat(ret, ".o");
            }
            
        } else if(!strcmp(key, "buildrule")) {
        
            if(br = grabpr->tmpl->getbuildrule(value)) {
                if(currentmakemode == mmlocal) {
                    ts = "echo \"motor: *** " + grabpr->name + "|" +
                    value + "\"; " + br->compile;
                } else {
                    ts = br->compile;
                }

                ret = strdup(ts.c_str());
            }
            
        } else if(!strcmp(key, "cutlibname")) {
        
            if(!strncmp(value, "lib", 3)) {
                ret = strdup(value+3);
                if(r = strstr(ret, ".a")) *r = 0;
            }
            
        } else if(!strcmp(key, "justfname")) {

            ret = strdup(justfname(value));
            
        } else if(!strcmp(key, "nospace")) {

            ret = strdup(value);
            for(r = ret; *r; r++)
            if(*r == ' ') *r = '_';

        } else if(!strcmp(key, "cutconnected")) {

            ret = strdup(value);
            if(grabpr->isconnectedpathname(ret)) ret[0] = 0;
            
        }
    }

    return ret;
}

void motorproject::exec_editload(int argc, const char **argv) {
    for(int i = 0; i < argc; i++) ed.openfile(argv[i], 0, true);
}

void motorproject::populateparselist(pparamslist *params) {
    int i, k;
    char buf[512], *r;
    projectfile *pf;
    filefolder *fold;
    linkedlist *flist;
    pparamslist *pl, *cplist;
    motorproject *connectedp;
    
    parser_svalue_reg(&svmakefile);
    parser_exec_reg("editload", &exec_editload);
    
    pparamslist_add("name", name.c_str(), params);
    pparamslist_add("version", version.c_str(), params);
    pparamslist_add("result", result.c_str(), params);
    pparamslist_add("rootdir", rootdir.c_str(), params);
    pparamslist_add("packoutdir", packgendir.c_str(), params);
    pparamslist_add("prefix", "/usr/local/", params);
    pparamslist_add("compileroptions", compoptions.c_str(), params);
    pparamslist_add("linkeroptions", linkoptions.c_str(), params);
    pparamslist_add("makecommand", altermake.c_str(), params);
    pparamslist_add("aftermake", aftermake.c_str(), params);
    pparamslist_add("installcommand", packinstcmd.c_str(), params);
    if(leavemakef) pparamslist_add("leavemakef", "", params);
    pparamslist_add("cureditfile", absfname(ed.getcurrentfname()).c_str(), params);

    setpwent();
    struct passwd *p = getpwuid(getuid());
    r = strchr(p->pw_gecos, ',');
    if(r) *r = 0;
    pparamslist_add("author", p->pw_gecos, params);
    endpwent();

    if(pconnected) pparamslist_add("putconnected", "", params);
    if(pautoconf) pparamslist_add("useautoconf", "", params);
    if(paddinfo) pparamslist_add("putprojectinfo", "", params);

    for(i = 0; i < tmpl->getfoldercount(); i++) {
        fold = tmpl->getfolder(i);
        flist = getfiles(fold);

        switch(fold->getelem()) {
            case feproj:
                pl = pparamslist_add_list(fold->getshortname().c_str(), params);

                for(k = 0; k < flist->count; k++) {
                    pf = (projectfile *) flist->at(k);
                    if(connectedp = plist.get(pf->getfname())) {
                        cplist = pparamslist_add_list(pf->getfname().c_str(), pl);
                        connectedp->populateparselist(cplist);
                    }
                }

                break;
                    
            default:
                pparamslist_add_array(fold->getshortname().c_str(), 0, 0, params);

                for(k = 0; k < flist->count; k++)
                pparamslist_add_array_value(
                    fold->getshortname().c_str(),
                    ((projectfile *) flist->at(k))->getfname().c_str(),
                    params
                );
                
                break;
        }
    }

    pl = pparamslist_add_list("inst", params);
    
    for(i = 0; i < getinstcount(); i++) {
        installrule *ir = getinst(i);
        cplist = pparamslist_add_list(ir->getfname().c_str(), pl);
        pparamslist_add("fname", ir->getfname().c_str(), cplist);
        pparamslist_add("dest", ir->getdest().c_str(), cplist);
    }
    
    grabpr = this;
}

void motorproject::genconnectedmakefiles(makenamemode mm = mmlocal) {
    int i, k;
    filefolder *f;
    linkedlist *flist;
    motorproject *mp;
    projectfile *pf;
    
    for(i = 0; i < tmpl->getfoldercount(); i++) {
        if((f = tmpl->getfolder(i))->getelem() == feproj) {
            flist = getfiles(f);
            
            for(k = 0; k < flist->count; k++) {
                pf = (projectfile *) flist->at(k);
                if(mp = plist.get(pf->getfname())) mp->genmakefile(mm);
            }
        }
    }
}

void motorproject::genmakefile(makenamemode mm = mmlocal) {
    pparamslist *params = pparamslist_create();
    FILE *f;

    populateparselist(params);
    if(mm == mmlocal) pparamslist_add("inmotor", "", params);
    currentmakemode = mm;

    if(f = fopen(getmakename().c_str(), "w")) {
        tmpl->writemakefile(params, f);
        fclose(f);
    }

    pparamslist_free(params);
}

void motorproject::getdebug() {
    vector<breakpoint>::iterator bi;
    vector<watchexpr>::iterator wi;

    dpoints.clear();

    for(bi = find(deb.bpoints.begin(), deb.bpoints.end(), true);
        bi != deb.bpoints.end();
        bi = find(bi+1, deb.bpoints.end(), true)
    ) {
        dpoints.push_back(debugpoint(dpbreakpoint,
            bi->fname + "\t" + i2str(bi->line)));
    }

    for(wi = deb.watches.begin();
        wi != deb.watches.end();
        wi++
    ) {
        dpoints.push_back(debugpoint(dpwatch, wi->name));
    }
}

void motorproject::putdebug() {
    vector<debugpoint>::iterator di;

    deb.clear();
    for(di = dpoints.begin(); di != dpoints.end(); di++) di->todebugger();
}

void motorproject::migratefiles(motortemplate *newt) {
    projectfile *pf;

    vector<string> fnames;
    vector<string>::iterator i;

    tmpl = newt;

    while(getfilecount()) {
        pf = getfile(0);
        fnames.push_back(pf->getfname());
        removefile(pf);
    }

    for(i = fnames.begin(); i != fnames.end(); i++) addfile(*i);
}

void motorproject::generatesource(linkedlist *ngen = 0) {
    pparamslist *params = pparamslist_create();
    char *resfn;
    string fname;
    FILE *f;
    int i;
    bool fin;

    chdir();
    populateparselist(params);
    
    for(i = 0, fin = false; !fin && (i < tmpl->getftemplcount()); i++) {
        if(ngen)
        if(!in_numberlist(ngen, i+1)) continue;

        fname = tmpl->getftempl(i);
        
        resfn = sstrparse(fname.c_str(), params);
        resfn[strlen(resfn)-5] = 0;

        if(!access(resfn, F_OK))
        switch(core.askf("YNC", _("File %s already exists. Overwrite it?"), resfn)) {
            case ANSWER_YES: break;
            case ANSWER_NO: continue;
            case ANSWER_CANCEL: fin = true; continue;
        }

        if(f = fopen(resfn, "w")) {
            fname.insert(0, "/");
            fname.insert(0, tmpl->gethome());
            
            strparse(fname.c_str(), fname.length(), params, f,
            PARSER_SRC_FILE, PARSER_FLOAD_MMAP);
            
            fclose(f);
            addfile(resfn);
        }
        
        delete resfn;
    }

    pparamslist_free(params);
}

bool motorproject::cvsenabled() {
    return !cvsmodule.empty() && !cvsroot.empty();
}

bool motorproject::modified() {
    int i, restime = 0, curtime = time(0);
    struct stat st;
    motorproject *pr;
    projectfile *pf;

    if(!stat(absfname(result).c_str(), &st)) restime = st.st_mtime;
    
    for(i = 0; i < getfilecount(); i++) {
        pf = getfile(i);

        switch(pf->getfolder()->getelem()) {
            case fefile:
                if(!stat(absfname(pf->getfname()).c_str(), &st))
                if(st.st_mtime > restime)
                if(st.st_mtime <= curtime) return true;
                break;
            case feproj:
                if(pr = plist.get(pf->getfname()))
                if(pr->modified()) return true;
                break;
        }
    }

    return false;
}

void *motorproject::foreachfile(listforeachfunc *lff, void *arg) {
    return files.foreach(lff, arg);
}

