#include "tagbrowser.h"
#include "project.h"
#include "edit.h"
#include "ui.h"

void tagbrowser::exec(string s = "") {
    string fname = (string) getenv("HOME") + "/.motor/tags." + proj->name;
    int lineno;

    if(pathfind("etags", getenv("PATH") ? getenv("PATH") : "").empty()) {
        core.bottom(_("Unable to run Exuberant Ctags (etags executable)"));
        return;
    }

    checktags(fname);

    if(s.size()) {
        setsearch(s);
        modified = false;
    }

    if(!tags.empty()) {
        core.bottom(_("Symbol browser: type to find, ~Del~ clears, press ~Enter~ to go to source"));

        if(open(fname, lineno)) {
            ed.openfile(fname, lineno);
        }
        
        core.bottom();
    } else {
        core.bottom(_("No symbols found"));
    }
}

bool tagbrowser::checktags(string fname) {
    bool ret = true;
    static bool ran = false;

    if(!ran || access(fname.c_str(), F_OK) || ed.modified()) {
        ed.saveall();
        core.bottom(_("Executing Exuberant Ctags.."));
        
        pparamslist *params = pparamslist_create();
        proj->populateparselist(params);
        pparamslist_add("etags_fname", fname.c_str(), params);

	char *tagcmd = sstrparse(proj->tmpl->getetagscmd().c_str(), params);
	string cmd = (string) tagcmd + " >/dev/null 2>&1";
        int r = system(cmd.c_str());

        pparamslist_free(params);
        ran = true;
        read(fname);
        delete tagcmd;

        ret = (r != -1) && (r != 127);
    }

    return ret;
}

void tagbrowser::cleanup(string pname) {
    string fname = (string) getenv("HOME") + "/.motor/tags." + pname;
    unlink(fname.c_str());
}

// --------------------------------------------------------------------------

tagitem::tagitem(string buf) {
    int i, pos;
    string token;

    for(i = 0; buf.size(); i++) {
        if((pos = buf.find("\t")) == -1) pos = buf.size();
        token = buf.substr(0, pos);
        buf.replace(0, pos+1, "");
        
        switch(i) {
            case 0: name = token; break;
            case 1: fname = token; break;
            case 2: lineno = atol(token.c_str()); break;
            case 3: kind = token[0]; break;
            case 4: info = token; break;
        }
    }
}

tagitem::~tagitem() {
}

string tagitem::getname() {
    return name;
}

string tagitem::getfname() {
    return fname;
}

string tagitem::getinfo() {
    return info;
}

char tagitem::getkind() {
    return kind;
}

int tagitem::getlineno() {
    return lineno;
}

// ----------------------------------------------------------------------------

tagbrowser::tagbrowser() {
}

tagbrowser::~tagbrowser() {
}

void tagbrowser::set(int nx1, int ny1, int nx2, int ny2, int nwcolor,
int nncolor, int nscolor, int nhcolor, int nwoptions) {
    x1 = nx1, y1 = ny1, x2 = nx2, y2 = ny2;
    wcolor = nwcolor, ncolor = nncolor, scolor = nscolor;
    hcolor = nhcolor, woptions = nwoptions;
    listmode = lminfo;
}

int tagbrowser::mkeys(verticalmenu *m, int k) {
    string oldsearch = browser.search;

    switch(k) {
        case KEY_BACKSPACE:
	    if(!browser.search.empty()) {
		browser.search = oldsearch.substr(0, oldsearch.size()-1);
	    }
            break;

        case KEY_DC: browser.search = ""; break;

        default:
            if(isprint(k)) {
                if(!browser.modified) {
                    browser.search = "";
                    browser.modified = true;
                }
                
                browser.search += k;
            }
            break;
    }

    if(oldsearch != browser.search) {
        if((browser.search.size() > oldsearch.size()) && !m->getcount()) {
            browser.input();
        } else {
            browser.makemenu(m);
        }
    }

    return -1;
}

void tagbrowser::makemenu(verticalmenu *m) {
    int i, len, k;
    string s, skind, sl;
    vector<tagitem>::iterator t;

    m->clear();

    for(t = tags.begin(), k = 0; t != tags.end(); t++, k++)
    if(!strncmp(t->getname().c_str(), search.c_str(), search.size())) {
        s = " " + t->getname();
        sl = proj->tmpl->gettagname(t->getkind());

        switch(listmode) {
            case lminfo:
                skind = t->getinfo();
                if(skind.size() && (skind[skind.size()-1] != ':')) {
                    if(sl.size()) sl += ", ";
                    sl += skind;
                }
                
                break;
                
            case lmfname:
                if(!t->getfname().empty()) {
                    if(sl.size()) sl += ", ";
                    sl += justfname(t->getfname().c_str());
                }
                
                break;
        }

        len = w->x2-w->x1-s.size()-sl.size()-2;

        if(len < 2) {
            len = 2;
            sl.resize(w->x2-w->x1-s.size()-4);
        }

        s += skind.assign(len, ' ') + sl;
        m->additem(0, (void *) k, s);
    }

    m->redraw();
    input();
}

void tagbrowser::input() {
    attrset(wcolor);
    mvhline(w->y2-1, w->x1+2, ' ', w->x2-w->x1-2);
    w->write(2, w->y2-w->y1-1, hcolor, search);
}

bool tagbrowser::open(string &fname, int &lineno) {
    static int mpos = 0;
    int n, k;
    bool ret = false;
    w = new textwindow(x1, y1, x2, y2, wcolor, woptions, hcolor, " Symbols ");

    w->open();
    w->separatey(w->y2-w->y1-2);

    verticalmenu m(w->x1+1, w->y1+1, w->x2, w->y2-2, ncolor, scolor);

    m.otherkeys = &mkeys;

    makemenu(&m);
    m.setpos(mpos);
    
    if(ret = (n = m.open())) {
        k = (int) m.getref(n-1);
        fname = tags[k].getfname();
        lineno = tags[k].getlineno();
        mpos = n-1;
    }

    w->close();
    delete w;
    return ret;
}

void tagbrowser::parseline(string buf) {
    if(buf.size() && strncmp(buf.c_str(), "!_TAG_", 6)) {
        tags.push_back(tagitem(buf));
    }
}

void tagbrowser::read(string fname) {
    ifstream f(fname.c_str());
    string s;

    tags.empty();

    if(f.is_open()) {
        while(!f.eof()) {
            getline(f, s);
            parseline(s);
        }
        f.close();
    }
}

void tagbrowser::setsearch(string nsearch) {
    search = nsearch;
}

string tagbrowser::getsearch() {
    return search;
}

