#include "templ.h"
#include "conf.h"

motortemplate *curtempl = 0;

motortemplate::motortemplate() {
    ftempls.freeitem = &charpointerfree;
    tagnames.freeitem = &charpointerfree;
}

motortemplate::~motortemplate() {
}

buildrule *motortemplate::getbuildrule(string fname) {
    return (buildrule *) buildrules.find((void *) fname.c_str(), &findbuildrule);
}

int motortemplate::getdistcount() {
    return dists.count;
}

int motortemplate::getftemplcount() {
    return ftempls.count;
}

distmethod *motortemplate::getdist(int n) {
    void *p = dists.at(n);
    return p ? (distmethod *) p : 0;
}

string motortemplate::getftempl(int n) {
    return (const char *) ftempls.at(n);
}

int motortemplate::getfoldercount() {
    return folders.count;
}

filefolder *motortemplate::getfolder(string name) {
    return (filefolder *) folders.find((void *) name.c_str(), &findfolderbyname);
}

filefolder *motortemplate::getfolder(int n) {
    return (filefolder *) folders.at(n);
}

string motortemplate::getname() {
    return name;
}

string motortemplate::gethome() {
    return home;
}

string motortemplate::getresultrule() {
    return resultrule;
}

string motortemplate::getetagscmd() {
    return etagscmd;
}

string motortemplate::gettagname(char kind) {
    int i;
    char *p;
    string ret;
    ret += kind;
    
    for(i = 0; i < tagnames.count; i++) {
        p = (char *) tagnames.at(i);
        if(kind == p[0]) {
            ret = p+1;
            break;
        }
    }

    return ret;
}

void motortemplate::load(string templname) {
    name = templname;
    
    home = (string) getenv("HOME") + "/.motor/templates/" + templname;
    if(access(home.c_str(), F_OK)) home = (string) SHAREDIR + "/motor/templates/" + templname;

    readbuildrules();
    readfolders();
    readmakefile();
    readdistmethods();
    readsyntax();
    readfiletemplates();
}

void motortemplate::readmakefile() {
    FILE *f;
    struct stat sb;
    string fname = home + "/Makefile";

    if(f = fopen(fname.c_str(), "r")) {
        fstat(fileno(f), &sb);
        char *buf = new char[sb.st_size+1];
        fread(buf, sb.st_size, 1, f);
        buf[sb.st_size] = 0;
        makefile = buf;
        delete buf;
        fclose(f);
    }
}

void motortemplate::readdistmethods() {
    DIR *d;
    FILE *f;
    struct dirent *ent;
    string fname;
    char *p;
    regex_t r;
    regcomp(&r, "\\.Dist$", REG_EXTENDED);

    if(d = opendir(home.c_str())) {
        while(ent = readdir(d))
        if(!regexec(&r, ent->d_name, 0, 0, 0)) {
            fname = home + "/" + ent->d_name;
            p = strdup(ent->d_name);
            p[strlen(p)-5] = 0;

            if(p[0])
            if(f = fopen(fname.c_str(), "r")) {
                dists.add(new distmethod(p, f));
                fclose(f);
            }

            delete p;
        }
        closedir(d);
    }

    regfree(&r);
}

void motortemplate::readbuildrules() {
    buildrule *br = 0;
    parserule *pr;
    string porder, sect, buf;
    int i, npos;

    string fname = home + "/Buildrules";
    ifstream f(fname.c_str());

    if(f.is_open()) {
        while(getconf(sect, buf, f)) {
	    if(sect.substr(0, 5) == "rule ") {
                if(br)
                if(!br->fname.empty())
                if(sect.substr(5) != br->fname) {
                    buildrules.add(br);
                    br = 0;
                }
                
                if(!br) {
                    br = new buildrule;
                    br->parse = new linkedlist;
                }

                if(br->fname.empty()) br->fname = sect.substr(5);
                br->compile += buf;

            } else if(sect == "parse") {
                if(br) {
                    pr = new parserule;
                    pr->mask = buf;

                    for(npos = pr->mask.size()-1; npos; npos--) {
                        if(strchr(" \t", pr->mask[npos])) {
                            porder = pr->mask.substr(npos+1);
                            pr->mask.resize(npos);
                            break;
                        }
                    }
                    
                    for(npos = 0; npos < porder.size(); npos++)
                    switch(porder[npos]) {
                        case 'F': pr->nfname = npos; break;
                        case 'L': pr->nline = npos; break;
                        case 'D': pr->ndescr = npos; break;
                        case 'E': pr->iserror = true; break;
                        case 'W': pr->iserror = false; break;
                    }
                    
                    br->parse->add(pr);
                }
            } else if(sect == "resultname") {
                resultrule = buf;
            }
        }

        if(br) buildrules.add(br);
	f.close();
    }
}

void motortemplate::readsyntax() {
    string fname = home + "/Browser", sect, buf;
    ifstream f(fname.c_str());

    if(f.is_open()) {
        while(getconf(sect, buf, f)) {
            if(sect == "etags") {
                if(etagscmd.size()) etagscmd += "\n";
                etagscmd += buf;
            } else if(sect == "tagnames") {
                if(buf.size() > 2) {
                    while((buf.size() > 2) && isspace(buf[1])) buf.replace(1, 1, "");
                    tagnames.add(strdup(buf.c_str()));
                }
            }
        }
        
        f.close();
    }
}

int motortemplate::readfiletemplatesftw(const char *file, const struct stat *sb, int flag) {
    if((flag == F_OK) && !strcmp(file+strlen(file)-5, ".tmpl")) {
        file += curtempl->home.length();
        for(; file[0] == '/'; file++);
        curtempl->ftempls.add(strdup(file));
    }
    
    return 0;
}

void motortemplate::readfiletemplates() {
    curtempl = this;
    ftw(home.c_str(), &readfiletemplatesftw, 5);
}

void motortemplate::addfolder(filefolder *fold, linkedlist &collect) {
    int i;
    string buf;
    
    if(fold) {
        if(fold->getelem() == feproj) {
            for(i = 0; i < collect.count; i++) {
                if(!buf.empty()) buf += "|";
                buf += (char *) collect.at(i);
            }
            
            if(!buf.empty()) fold->setmask(buf.c_str());
        }
        folders.add(fold);
    }
}

void motortemplate::readfolders() {
    string fname = home + "/Folders", sect, prevsect, buf, param;
    filefolder *fold = 0;
    ifstream f(fname.c_str());
    const char *p;
    int i;

    linkedlist collect;
    collect.freeitem = &charpointerfree;

    if(f.is_open()) {
        while(getconf(sect, buf, f)) {
            if((sect != prevsect) || !prevsect.size()) {
                addfolder(fold, collect);
                fold = new filefolder;
                fold->setshortname(sect);
                collect.empty();
            }

            prevsect = sect;
            param = getword(buf);

            if(param == "element") {

                if(buf == "file") fold->setelem(fefile); else
                if(buf == "symbol") fold->setelem(fesymb); else
                if(buf == "dir") fold->setelem(fedir); else
                if(buf == "proj") fold->setelem(feproj);
                
            } else if(param == "take") {

                switch(fold->getelem()) {
                    case feproj:
                        if(p = strqstr(buf.c_str(), "from", "\"")) {
                            param = buf;
                            param.replace(0, p-buf.c_str(), "");
                            getword(param);
                            
                            if(!collect.find((void *) param.c_str(), &stringcompare)) {
                                collect.add(strdup(param.c_str()));
                            }
                        }
                        break;
                }

                fold->addtake(buf);
                
            } else if(param == "mask") {
                fold->setmask(buf);
            } else if(param == "reject") {
                fold->setreject(buf);
            } else if(param == "name") {
                fold->setname(buf);
            }
        }

        addfolder(fold, collect);
        f.close();
    }
}

void motortemplate::writemakefile(pparamslist *params, FILE *f) {
    if(!makefile.empty()) fstrparse(makefile.c_str(), params, f);
}

int motortemplate::findfolderbyname(void *fname, void *folder) {
    return (string) ((const char *) fname) != ((filefolder *) folder)->getshortname();
}

int motortemplate::findbuildrule(void *fname, void *rule) {
    regex_t r;
    buildrule *br = (buildrule *) rule;
    int ret = 1;
    
    if(!regcomp(&r, br->fname.c_str(), REG_EXTENDED)) {
        ret = regexec(&r, (char *) fname, 0, 0, 0);
        regfree(&r);
    }

    return ret;
}

