#include "ui.h"
#include "conf.h"
#include "project.h"
#include "compile.h"
#include "manager.h"
#include "dist.h"
#include "edit.h"
#include "grep.h"
#include "tagbrowser.h"
#include "tlist.h"
#include "debug.h"

const char *askres[] = {_("yes"), _("no"), _("cancel"), _("all")};

CMotorCore::CMotorCore() {
    wwatch = 0;
    watchstart = 0;

    kinterface();
    setsignals();
}

CMotorCore::~CMotorCore() {
    delete il;
    delete fm;
}

void CMotorCore::init() {
    string localedir = (string) SHAREDIR + "/locale";
    setlocale(LC_ALL, "");
    bindtextdomain("motor", localedir.c_str());
    textdomain("motor");

    ed.init();
    
    fm = new filemanager(1, 2, (int) (COLS*0.7), (int) (LINES*0.75),
    conf.getcolor(cp_menu), conf.getcolor(cp_menuhl), 0,
    conf.getcolor(cp_menusel), conf.getcolor(cp_menufr));
    
    il = new textinputline(conf.getcolor(cp_input));

    hmenu = horizontalmenu(1, 0, conf.getcolor(cp_menu),
    conf.getcolor(cp_menusel), conf.getcolor(cp_menufr));

    hmenu.additem(_(" File "));
    hmenu.additem(_(" Edit "));
    hmenu.additem(_(" Project "));
    hmenu.additem(_(" CVS "));
    hmenu.additem(_(" Debug "));
    hmenu.additem(_(" Window "));
    
    verticalmenu *m = hmenu.pulldown(0);
    m->additem(_(" Load..         ^O"));
    m->additem(_(" Save           F2"));
    m->additem(_(" Save as.."));
    m->additem(_(" Save all"));
    m->additem(_(" Close          ^D"));
    m->additem(_(" Generate.."));
    m->addline();
    m->additem(_(" Motor settings.."));
    m->additem(_(" External keys.."));
    m->addline();
    m->additem(_(" Quit           ^X"));

    m = hmenu.pulldown(1);
    m->additem(_(" Toggle mark         F3"));
    m->additem(_(" Cut                A-X"));
    m->additem(_(" Copy               A-C"));
    m->additem(_(" Paste              A-V"));
    m->addline();
    m->additem(_(" Toggle ins/overw   Ins"));
    m->addline();
    m->additem(_(" Global find        A-F"));
    m->additem(_(" Global find again  A-G"));
    m->addline();
    m->additem(_(" Find                ^F"));
    m->additem(_(" Find again          ^G"));
    m->additem(_(" Replace             ^R"));
    m->additem(_(" Go to line..       A-L"));
    m->addline();
    m->additem(_(" Shft block fwd   A-Tab"));
    m->additem(_(" Shft block back  A-Bsp"));
    m->addline();
    m->additem(_(" Undo                ^U"));

    m = hmenu.pulldown(2);
    m->additem(_(" Projects registry"));
    m->addline();
    m->additem(_(" Files..             F11"));
    m->additem(_(" Directories..      ^F11"));
    m->additem(_(" Project settings  S-F11"));
    m->addline();
    m->additem(_(" Build..              F9"));
    m->additem(_(" Clean              S-F9"));
    m->additem(_(" Make a package"));
    m->addline();
    m->additem(_(" Find a symbol..     A-?"));

    m = hmenu.pulldown(3);
    m->additem(_(" CVS check in/out"));
    m->additem(_(" Import into CVS"));
    m->additem(_(" Tag a release"));
    m->additem(_(" Check out specific release"));
    m->additem(_(" Current file modification history"));

    m = hmenu.pulldown(4);
    m->additem(_(" Run program            F12"));
    m->additem(_(" Load core dump.."));
    m->additem(_(" Arguments..          S-F12"));
    m->additem(_(" Reset program         ^F12"));
    m->additem(_(" Call stack..           A-S"));
    m->addline();
    m->additem(_(" Set/Clear breakpoint    ^B"));
    m->additem(_(" Breakpoints            A-B"));
    m->addline();
    m->additem(_(" Add watch               ^W"));
    m->additem(_(" Watches                A-W"));
    m->additem(_(" Evaluate/Modify        A-E"));
    m->addline();
    m->additem(_(" Trace into              F7"));
    m->additem(_(" Step over               F8"));
    m->additem(_(" Go to cursor            F4"));
}

void CMotorCore::shutdown() {
    kendinterface();
    for(int i = 0; i < LINES; i++, printf("\n"));
}

verticalmenu *CMotorCore::getpulldown(int n) {
    return hmenu.pulldown(n);
}

void CMotorCore::sighandler(int signum) {
    int status;

    switch(signum) {
	case SIGCHLD:
	    while(wait3(&status, WNOHANG, 0) > 0);
	    break;
    }

    core.setsignals();
}

void CMotorCore::setsignals() {
    signal(SIGCHLD, &sighandler);
    signal(SIGINT, &sighandler);
}

void CMotorCore::restorestate() {
}

bool CMotorCore::build() {
    bool success = true;
    ed.saveall(false);

    bottom(_("Writing Makefile"));
    proj->genconnectedmakefiles();
    proj->genmakefile();
    
    bottom(_("Executing make"));
    success = cmp.runmake(mmlocal, "all");
    proj->clearmakefiles();

    if(success && cmp.nfiles) reset();

    if(success && !cmp.nerr) {
        if(cmp.nfiles) {
	    if(!cmp.nwarn) {
		bottom(_("Compiled successfully"));
	    } else {
		bottomf(_("Compiled successfully, %d warnings, ~F6~ to see output"), cmp.nwarn);
            }
        } else {
	    bottomf(_("%s is up to date"), proj->result.c_str());
        }
    } else {
	bottomf(_("%d errors, %d warnings, ~F6~ to see output, ~Ctrl-E~ next message"), cmp.nerr, cmp.nwarn);
    }

    if(!success || cmp.nerr) cmp.show();
    return success;
}

void CMotorCore::makeclean() {
    proj->genconnectedmakefiles();
    proj->genmakefile();
    core.bottom(_("executing make clean"));
    cmp.runmake(mmlocal, "clean");
    unlink(proj->getmakename().c_str());
    core.bottom(_("make clean: done"));
}

int CMotorCore::execmenu(texteditor *ref) {
    static int h = 0, p = 0, ret = 0;

    if(core.hmenu.open(&h, &p))
    switch(h) {
        case 1:
	    switch(p) {
    	        case 1: ed.load(); break;
    		case 2: ed.save(); break;
		case 3: ed.saveas(); break;
		case 4: ed.saveall(false); break;
    		case 5: ed.closewindow(0); break;
    		case 6: mgr.generatefiles(false); break;
        	case 8: conf.execute(); break;
                case 9: conf.extkeys.showlist(); break;
		case 11: ret = -1; break;
    	    }
    	    break;
        case 2:
	    switch(p) {
    	        case  1: ed.switchmark(); break;
		case  2: ed.editoperation(edcut); break;
	        case  3: ed.editoperation(edcopy); break;
		case  4: ed.editoperation(edpaste); break;
    	        case  6: ed.switchins(); break;
        	case  8: grep.exec(); break;
    	        case  9: grep.nextresult(); break;
                case 11: ed.findtext(false); break;
                case 12: ed.findtext(true); break;
    	        case 13: ed.replace(); break;
    	        case 14: ed.gotoline(); break;
    	        case 16: ed.shift(SD_FORWARD); break;
    	        case 17: ed.shift(SD_BACKWARD); break;
	        case 19: ed.undo(); break;
            }
            break;
        case 3:
	    switch(p) {
	        case  1: mgr.list(); break;
	        case  3: mgr.files(); break;
    		case  4: mgr.dirs(); break;
	        case  5: mgr.settings(); break;
	        case  7: core.build(); break;
	        case  8: makeclean(); break;
	        case  9: dist.dialog(); break;
	        case 11: browser.exec(); break;
    	    }
            break;
        case 4:
            if(cvs.nocvs) {
                bottom(_("Can't find any CVS installed. Aborted."));
            } else {
                if((!proj->cvsmodule.empty() && !proj->cvsroot.empty()) || (p == 2))
	        switch(p) {
        	    case 1: cvscheck(cknone); break;
		    case 2: cvsimport(); break;
		    case 3: cvscommand(cktag); break;
		    case 4: cvsgetrelease(); break;
		    case 5: cvsfilehistory(); break;
                } else {
		    bottom(_("This project isn't under the CVS control"));
                }
            }
	    break;
        case 5:
	    switch(p) {
                case  1: run(); break;
                case  2: debugcore(); break;
	        case  3: setargs(); break;
	        case  4: reset(); break;
                case  5: callstack(); break;
	        case  7: breakpoint(); break;
	        case  8: openbreakpoints(); break;
	        case 10: addwatch(ed.atcursor()); break;
	        case 11: openwatch(true); break;
                case 12: evaluate(ed.atcursor()); break;
                case 14: debugproceed(true); break;
	        case 15: debugproceed(false); break;
                case 16: debugtocursor(); break;
            }
            break;
        case 6:
            switch(p) {
	        case 1: ed.switchwindow(+1); break;
	        case 2: ed.switchwindow(-1); break;
	        case 3: ed.windowlist(); break;
	        default:
	            ref->setfnum(p-5);
	            ed.status();
                    break;
	    }
            break;
    }

    ed.updatecursor();
    return ret;
}

int CMotorCore::ask(string options, string msg) {
    int i, c, ret, line;

    if(wwatch) line = (int) (LINES-LINES*0.3); else line = LINES-2;
    chtype **sbuf = savelines(0, line, COLS, line);
    
    attrset(conf.getcolor(cp_input));
    mvhline(line, 0, ' ', COLS);

    msg += " (";
    for(i = 0; i < options.size(); i++) {
        msg += i ? tolower(options[i]) : toupper(options[i]);
        msg += '/';
    }

    msg.resize(msg.size()-1);
    msg += ") ";

    kwriteat(0, line, msg.c_str(), conf.getcolor(cp_input));

    while(1) {
        c = toupper(getch());
        if(c == '\r') c = toupper(options[0]);
        if(options.find(c) != -1) break;
    }

    switch(c) {
        case 'Y': ret = ANSWER_YES; break;
        case 'N': ret = ANSWER_NO; break;
        case 'C': ret = ANSWER_CANCEL; break;
        case 'A': ret = ANSWER_ALL; break;
    }

    kwriteat(kwherex(), kwherey(), (char *) askres[ret], conf.getcolor(cp_input));
    restorelines(sbuf, 0, line, COLS, line);

    return ret;
}

int CMotorCore::askf(string options, const char *fmt, ...) {
    va_list ap;
    char buf[1024];

    va_start(ap, fmt);
    vsprintf(buf, fmt, ap);
    va_end(ap);

    return ask(options, buf);
}

string CMotorCore::inputfile(string initval, string text) {
    il->fm = fm;
    fm->onlydirs = false;
    il->fm->w->set_title(conf.getcolor(cp_menuhl), _(" enter to select a file, esc to cancel "));
    string ans = inputskel(initval, text);
    il->fm = 0;
    return ans;
}

string CMotorCore::inputdir(string initval, string text) {
    il->fm = fm;
    fm->onlydirs = true;
    il->fm->w->set_title(conf.getcolor(cp_menuhl), _(" space to select a directory, esc to cancel "));
    string ans = inputskel(initval, text);
    il->fm = 0;
    return ans;
}

string CMotorCore::input(string initval, string text) {
    return inputskel(initval, text);
}

string CMotorCore::inputskel(string initval, string text) {
    int line;
    int mright = COLS-text.size();

    if(wwatch) line = (int) (LINES-LINES*0.3); else line = LINES-2;
    chtype **sbuf = savelines(0, line, COLS, line);

    attrset(conf.getcolor(cp_input));
    mvhline(line, 0, ' ', COLS);
    kwriteatf(0, line, conf.getcolor(cp_input), "%s", text.c_str());

    if(il->fm) {
         mright -= 8;
         kwriteatf(COLS-8, line, conf.getcolor(cp_input), "[Ctrl-T]");
    }

    char *p = il->open(text.size(), line, initval.c_str(), mright, 2048);
    string ret = p;
    delete p;

    restorelines(sbuf, 0, line, COLS, line);
    return ret;
}

void CMotorCore::bottomf(const char *fmt, ...) {
    va_list ap;
    char buf[10240];
    
    va_start(ap, fmt);
    vsprintf(buf, fmt, ap);
    va_end(ap);

    bottom(buf);
}

void CMotorCore::bottom(string bstr = "") {
    int i;
    bool hl = false;

    attrset(conf.getcolor(cp_bottom));
    mvhline(LINES-1, 0, ' ', COLS);
    mvprintw(LINES-1, 0, "");

    for(i = 0; i < bstr.size(); i++) {
        if(bstr[i] == '~') {
            if(hl = !hl) attrset(conf.getcolor(cp_bottomhl));
            else attrset(conf.getcolor(cp_bottom));
        } else {
            printw("%c", bstr[i]);
        }
    }
    
    refresh();
}

void CMotorCore::exec() {
    int i;

    attrset(conf.getcolor(cp_bottom));
    mvhline(LINES-1, 0, ' ', COLS);
    mvhline(0, 0, ' ', COLS);

    if(!proj) mgr.list(); else {
        ed.editload();
        cvscheck(ckupdate);
        if(proj->watchactive) core.openwatch(false);
    }

    if(proj) {
        if(!proj->tmpl) {
            bottom(_("Template for this project not found"));
            
            if(proj->tmpl = tlist.select()) proj->migratefiles(proj->tmpl);
            else return;
        }

        while(1) {
            ed.status();

            if(ed.open() == TEXTEDITOR_NOFILES) {
                if(!proj) break;

                if(!proj->getfilecount()) {
                    if(!mgr.settings()) break;
                } else {
                    if(!mgr.files()) {
                        proj->edclear();
                        break;
                    }
                }
            } else {
                if(ed.editclose(CA_DONTSHOW)) break;
            }
        }

        if(proj) {
            proj->save();
            core.cvscheck(ckcommit);
        }
    }
}

void CMotorCore::redraw() {
    ed.redraw();
    ed.status();
    updatewatches();
}

void CMotorCore::help() {
    dialogbox db;
    string helptext;

    db.setwindow(new textwindow(0, 0, (int) (COLS*0.8), (int) (LINES*0.65),
        conf.getcolor(cp_menufr), TW_CENTERED,
        conf.getcolor(cp_menuhl), _(" Help ")));

    db.setbrowser(new textbrowser(conf.getcolor(cp_menu)));

    db.setbar(new horizontalbar(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel), _("Ok"), 0));

    helptext += _("F1           Display this help screen\n");
    helptext += _("F10          Activate menu\n");
    helptext += _("^X           Quit motor\n\n");
    helptext += _("F2           Save current file\n");
    helptext += _("^O           Load file\n");
    helptext += _("^D           Close current editor window\n");
    helptext += _("F5           Editor files..\n");
    helptext += _("^N           Next editor window\n");
    helptext += _("^P           Previous editor window\n\n");
    helptext += _("Ins          Toggle insert/overwrite mode\n");
    helptext += _("F3           Toggle block marking mode\n");
    helptext += _("Alt-X, ^Del  Cut selected text\n");
    helptext += _("Alt-C, ^Ins  Copy selected text\n");
    helptext += _("Alt-V, ^Ins  Paste previously copied text\n");
    helptext += _("Alt-Tab      Shift selected block forward\n");
    helptext += _("Alt-Backsp   Shift selected block backward\n\n");
    helptext += _("^U           Undo previous editor operation\n");
    helptext += _("^F           Find text in current file\n");
    helptext += _("^G           Find again\n");
    helptext += _("^R           Search and replace\n");
    helptext += _("Alt-L        Goto line..\n");
    helptext += _("Alt-F        Invoke internal Grepper tool\n");
    helptext += _("Alt-G        Go to the next match found with Grepper\n\n");
    helptext += _("Shift-F11    Project settings..\n");
    helptext += _("F11          Project files..\n");
    helptext += _("^F11         Project directories..\n\n");
    helptext += _("F9           Build a project\n");
    helptext += _("Shift-F9     Clean\n");
    helptext += _("F12          Run program\n");
    helptext += _("Shift-F12    Specify program arguments\n");
    helptext += _("Alt-?        Find a symbol definition in the source\n\n");
    helptext += _("F4           Go to cursor\n");
    helptext += _("F7           Debug mode: trace into\n");
    helptext += _("F8           Step over\n\n");
    helptext += _("^B           Set breakpoint at current line\n");
    helptext += _("Alt-B        Breakpoints..\n");
    helptext += _("^W           Add watch\n");
    helptext += _("Alt-W        Watches..\n");
    helptext += _("Alt-E        Evaluate/modify..\n\n");
    helptext += _("Note: If your terminal doesn't allow using Alt-keys or\n");
    helptext += _("      Fx keys, you can use ESC 'letter or number' instead.\n");
    helptext += _("      '^' char means the key should be pressed with Ctrl.");

    db.getbrowser()->setbuf(helptext.c_str());

    db.open();
    db.close();
}

