#include "ui.h"
#include "project.h"
#include "conf.h"
#include "edit.h"

void CMotorCore::cvscheck(cvscommandkind ck) {
    bool finished = false, toupdate, tocommit, res;
    vector<cvsupdated> chlist;
    dialogbox db;
    string text;
    int i, b;

    if(!proj || !proj->cvsenabled()) return;
    if((ck == ckupdate) && !conf.autocvsupdate) return;
    if((ck == ckcommit) && !conf.autocvscommit) return;
    
    if(cvs.nocvs) {
        bottom(_("Can't find any CVS installed. Aborted."));
        return;
    }

    bottom(_("CVS: looking for modified files"));

    while(!finished) {
        cvs.status(proj->rootdir, chlist);
        if(!(finished = cvs.result == rkok)) {
            finished = (core.ask("YN", _("cvs status command wasn't successfull. Retry?")) == ANSWER_NO);
        }
    }

    if((cvs.result == rkok) && !chlist.empty()) {
        db.setwindow(new textwindow(0, 0, (int) (COLS*0.8), (int) (LINES*0.65),
            conf.getcolor(cp_menufr), TW_CENTERED,
            conf.getcolor(cp_menuhl), _(" Modified files ")));

        db.setbrowser(new textbrowser(conf.getcolor(cp_menu)));

        db.setbar(new horizontalbar(conf.getcolor(cp_menu),
            conf.getcolor(cp_menusel),
            _("Update"), _("Tag a release"), _("Commit"), _("Continue"), 0));

        switch(ck) {
            case ckupdate: db.getbar()->item = 0; break;
            case ckcommit: db.getbar()->item = 2; break;
            default: db.getbar()->item = 3; break;
        }

        for(finished = false; !finished; ) {
            text = "";
            toupdate = cvsfilltext(chlist, text, mkremote);
            tocommit = cvsfilltext(chlist, text, mklocal);
            
            res = cvsfilltext(chlist, text, mklocaladded);
            tocommit = tocommit || res;
            res = cvsfilltext(chlist, text, mklocalremoved);
            tocommit = tocommit || res;

            if(text.empty()) break;
            db.getbrowser()->setbuf(text.c_str());

            finished = !db.open(i, b);

            if(!finished)
            switch(b) {
                case 0: cvscommand(ckupdate); break;
                case 1: cvscommand(cktag); break;
                case 2: cvscommand(ckcommit); break;
                default: finished = true; break;
            }

            if(!finished) cvs.status(proj->rootdir, chlist);
        }

        db.close();
    }

    bottom(_("CVS: checking is done"));
}

void CMotorCore::cvsgetrelease() {
    int n, i, b, welem = 0;
    dialogbox db;

    vector<cvsrelease> lst;
    vector<cvsrelease>::iterator ir;
    vector<string>::iterator is;

    bottom(_("Retreiving release list"));

    db.setwindow(new textwindow(0, 0, (int) (COLS*0.8), (int) (LINES*0.65),
        conf.getcolor(cp_menufr), TW_CENTERED));

    db.setmenu(new verticalmenu(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel)));

    db.setbar(new horizontalbar(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel), _("Check out"), 0));

    cvs.statusv(proj->rootdir, lst);

    bool finished = false, whole = true, owhole = false;

    while(!finished) {
        if(owhole != whole) {
            db.getmenu()->clear();

            if(whole) {
                db.getwindow()->set_title(conf.getcolor(cp_menuhl), _(" Module releases "));
                for(ir = lst.begin(); ir != lst.end(); ir++)
                    db.getmenu()->additem(" " + ir->name + " ");
            } else {
                db.getwindow()->set_title(conf.getcolor(cp_menuhl), _(" Release %s "), ir->name.c_str());
                db.getmenu()->additemf(conf.getcolor(cp_menuhl), 0, _(" .. Back to release list"));
                db.getmenu()->additemf(conf.getcolor(cp_menuhl), 0, _(" .. Check out whole the release"));
                db.getmenu()->setpos(0);

                for(is = ir->files.begin(); is != ir->files.end(); is++)
                    db.getmenu()->additem(" " + *is + " ");
            }
        }

        owhole = whole;

        finished = !db.open(n, b);

        if(!finished) {
            if(whole) {
                ir = lst.begin()+n-1;
                whole = false;
                welem = db.getmenu()->getpos();
            } else {
                if(n == 1) {
                    whole = true;
                    db.getmenu()->setpos(welem);
                } else {
                    cvscheckoutver(ir->name, n == 2 ? "" : ir->files[n-3]);
                }
            }
        }
    }
    
    db.close();
    ed.redraw();
}

void CMotorCore::cvsfilehistory() {
    int n, i, b;
    string fname = proj->relativefname(ed.getcurrentfname());
    dialogbox db;

    vector<cvslog> lst;
    vector<cvslog>::iterator il;

    bottom(_("Retrieving modification history"));

    db.setwindow(new textwindow(0, 0, (int) (COLS*0.8), (int) (LINES*0.65),
        conf.getcolor(cp_menufr), TW_CENTERED,
        conf.getcolor(cp_menuhl), _(" File modification history ")));
    
    db.setmenu(new verticalmenu(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel)));

    cvs.log(proj->rootdir, fname, lst);

    for(il = lst.begin(); il != lst.end(); il++)
        db.getmenu()->additemf(" %-12s %-20s %s",
            il->author.c_str(), il->date.c_str(),
            il->log.c_str());

    db.setbar(new horizontalbar(conf.getcolor(cp_menu),
        conf.getcolor(cp_menusel), _("View log"), _("Check out"), 0));

    while(db.open(n, b)) {
        switch(b) {
            case 0: cvslogshow(lst[n-1]); break;
            case 1: cvscheckoutver(lst[n-1].revision, fname); break;
        }
    }
    
    db.close();
    ed.redraw();
}

void CMotorCore::cvsimport() {
    int n;
    bottom(_("Select a repository to import the project into"));
    
    if(n = cvsrootselect(false)) {
        string ans = core.input(proj->cvsmodule, _("path in repository to import the module into: "));
        
        if(!ans.empty()) {
            string selrepos = conf.getrepos(n-1);
            bool finished = false;

            cvs.setrepos(selrepos);
            ed.editclose(CA_DONTSHOW | CA_DONTASK);
            core.makeclean();
	    bottom(_("Importing into the CVS repository"));
            
            while(!finished) {
                cvs.import(proj->rootdir, ans.c_str());
        
                if(cvs.result != rkok) {
	           finished = (core.ask("YN", _("cvs import command wasn't successfull. Retry?")) == ANSWER_NO);
                } else {
                   finished = true;

                   proj->cvsroot = selrepos;
                   proj->cvsmodule = ans.c_str();
                   cvs.checkout(proj->cvsmodule, proj->rootdir);
                   
                   bottom(_("CVS import done"));
                }
            }

            ed.editload();
            cvs.setrepos(proj->cvsroot);
        }
    }
}

int CMotorCore::cvsrootselect(bool emptyitem = true) {
    int r = 0, i;
    
    if(conf.getreposcount() || emptyitem) {
        dialogbox db;
        
        db.setwindow(new textwindow(0, 0, (int) (COLS*0.85), (int) (LINES*0.75),
            conf.getcolor(cp_menufr), TW_CENTERED,
            conf.getcolor(cp_menuhl), _(" Select CVS repository ")));
            
        db.setmenu(new verticalmenu(conf.getcolor(cp_menu),
            conf.getcolor(cp_menusel)));

        if(emptyitem) db.getmenu()->additemf(_(" Disable version control"));

        for(i = 0; i < conf.getreposcount(); i++) {
            string rep = conf.getrepos(i);
            db.getmenu()->additemf(" %s", rep.c_str());
            
            if(proj)
            if(rep == proj->cvsroot)
                db.getmenu()->setpos(i+1);
        }

        if(!db.open(r, i)) i = 0;
        db.close();
    } else {
        core.bottom(_("No CVS repositories registered. Use ~\"Settings\"~ to add"));
    }

    return r;
}

void CMotorCore::cvslogshow(const cvslog &clog) {
    int i, b;
    dialogbox db;

    db.setwindow(new textwindow(0, 0, (int) (COLS*0.8), (int) (LINES*0.65),
        conf.getcolor(cp_menufr), TW_CENTERED,
        conf.getcolor(cp_menuhl), _(" %s by %s "),
        clog.date.c_str(), clog.author.c_str()));

    db.setbrowser(new textbrowser(conf.getcolor(cp_menu)));

    db.getbrowser()->setbuf(clog.log.c_str());
    db.open(i, b);
    db.close();
}

void CMotorCore::cvscommand(cvscommandkind ck) {
    bool tocommit = false, finished = false;
    string relname, comment;
    vector<cvsupdated> lst;

    switch(ck) {
        case ckupdate:
            bottom(_("Checking out latest modifications from the CVS repository"));
            break;
        case ckcommit:
            if(!cvscomment(comment)) {
                return;
            } else {
                bottom(_("Checking in locally modified files into the CVS repository"));
            }
            break;
        case cktag:
            bottom(_("Checking project status"));
            cvs.status(proj->rootdir, lst);

            tocommit =
                (find(lst.begin(), lst.end(), mklocal) != lst.end()) ||
                (find(lst.begin(), lst.end(), mklocaladded) != lst.end()) ||
                (find(lst.begin(), lst.end(), mklocalremoved) != lst.end());

            if(tocommit)
            if(core.ask("YN", _("local changes must be commited before tagging. Do you want to proceed?")) == ANSWER_NO)
                return;

            relname = core.input("", _("release name to tag: "));
            if(relname.empty()) return;

            if(tocommit) {
                ed.saveall(false);
                cvscommand(ckcommit);
            }
            
            bottom(_("Tagging the release"));
            break;
    }

    while(!finished) {
        switch(ck) {
            case ckcommit: cvs.commit(proj->rootdir, comment); break;
            case ckupdate: cvs.update(proj->rootdir); break;
            case cktag: cvs.tag(proj->rootdir, relname.c_str()); break;
        }
        
        if(cvs.result != rkok) {
	    finished = (core.ask("YN", _("cvs command wasn't successfull. Retry?")) == ANSWER_NO);
	} else {
            finished = true;
            switch(ck) {
                case ckcommit: bottom(_("CVS commit done")); break;
                case ckupdate: bottom(_("CVS update done")); break;
                case cktag: bottom(_("CVS tag done")); break;
            }
        }
    }
}

void CMotorCore::cvscheckoutver(string rel, string fname) {
    bool cvsfinish = false;
    ed.editclose(CA_DONTASK | CA_DONTSHOW);
            
    while(!cvsfinish) {
        bottom(_("Checking out"));
        cvs.getrelease(proj->rootdir, proj->cvsmodule, rel, fname);

        if(cvs.result != rkok) {
	    cvsfinish = (core.ask("YN", _("cvs command wasn't successfull. Retry?")) == ANSWER_NO);
        } else {
            cvsfinish = true;
            char buf[1024];
            
            if(fname.empty()) sprintf(buf, _("Release %s of the project checked out"), rel.c_str());
            else sprintf(buf, _("Release %s of the file checked out"), rel.c_str(), fname.c_str());

            bottom(buf);
        }
    }

    ed.editload();
}

bool CMotorCore::cvscomment(string &ret) {
    texteditor ed;

    textwindow w(0, 0, (int) (COLS*0.8), (int) (LINES*0.65), conf.getcolor(cp_menufr), TW_CENTERED);
    w.set_title(conf.getcolor(cp_menuhl), _(" Comment the changes you made "));
    w.open();

    w.separatey(w.y2-w.y1-2);
    w.write(w.x2-w.x1-20, w.y2-w.y1-1, conf.getcolor(cp_menufr), "^X save  ESC cancel");
    w.write(w.x2-w.x1-20, w.y2-w.y1-1, conf.getcolor(cp_menuhl), "^X");
    w.write(w.x2-w.x1-11, w.y2-w.y1-1, conf.getcolor(cp_menuhl), "ESC");

    ed.setcoords(w.x1+1, w.y1+1, w.x2-1, w.y2-2);
    ed.addscheme(cp_menufr, cp_menufr, 0, 0);

    ed.otherkeys = &cvscommentkeys;
    ed.wrap = false;

    editok = false;
    ed.load("", strdup(""));
    ed.open();

    if(editok) {
        char *scomment = ed.save("\r\n");
        ret = scomment;
        delete scomment;
    }

    w.close();
    return editok;
}

bool CMotorCore::cvsfilltext(const vector<cvsupdated> &lst, string &text, cvsmodificationkind kind) {
    bool h = false;
    vector<cvsupdated>::const_iterator iu;

    for(iu = find(lst.begin(), lst.end(), kind); (iu = find(iu+1, lst.end(), kind)) != lst.end(); ) {
        if(!h) {
            switch(kind) {
                case mklocal: text += _(" Locally modified:"); break;
                case mkremote: text += _(" Remotely modified:"); break;
                case mklocaladded: text += _(" Locally added:"); break;
                case mklocalremoved: text += _(" Locally removed:"); break;
            }
            text += "\n";
            h = true;
        }

        text += "   " + iu->fname + "\n";
    }

    if(h) text += "\n";
    return h;
}

int CMotorCore::cvscommentkeys(texteditor *ed, int k) {
    switch(k) {
        case CTRL('x'):
            core.editok = true;
        case 27:
            return -1;
    }
    return 1;
}

